import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot und MouseInfo)
import java.util.*;
/**
 * Die Klasse Bauer definiert die Grundfunktionen des Bauern
 * Er hat ein Inventar, um Gegenstnde umhertragen zu knnen.
 * Diese kann er aufnehmen oder ablegen, sofern sie transportabel sind.
 * Die Giesskanne kann er benutzen, um Tomaten zu gieen.
 * Er kann Tomaten und Krner/Korn sen und ernten.
 * 
 * @author T. Schaller 
 * @version 08.11.2023
 */
public class Bauer extends Figur
{
    private ArrayList<Gegenstand> inventar= new ArrayList<Gegenstand>();;

    public Bauer() {
        super("Bauer");
    }

    /**
     * Gibt die Anzahl der Gegenstaende, die der Bauer traegt, zurueck.
     * @return Anzahl der Gegenstaende
     */
    public int getAnzahl() {
        return inventar.size();
    }

    /**
     * Gibt die Anzahl der Gegenstaende einer bestimmten Art zurueck.
     * @param name Art der Gegenstands ("Koerner", "Tomate", "Giesskanne", "Ei", "Blume", "Sense")
     * @return Anzahl der Gegenstaende
     */
    public int getAnzahl(String name) {
        int anz = 0;
        if (Gegenstand.istGegenstand(name)) {
            for (Gegenstand g : inventar) {
                if (g.getName().toUpperCase().equals(name.toUpperCase())) {
                    anz++;
                }
            }
        } else 
        {
            warne(""+name+" ist keine gltige Bezeichnung fr einen Gegenstand", this);
        }
        return anz;
    }

    /**
     * Gibt das erste Gegenstandsobjekt zurck, das der angegebenen Art entspricht.
     * @param name Art des gesuchten Gegenstands ("Koerner", "Tomate", "Giesskanne", "Ei", "Blume", "Sense")
     * @return Gegenstandsobjekt
     */
    public Gegenstand getGegenstandAusInventar(String name) {
        if (Gegenstand.istGegenstand(name)) {
            if (inventar == null) return null;
            for(Gegenstand g: inventar) {
                if (g.getName().toUpperCase().equals(name.toUpperCase())) {
                    return g;
                }
            }
        }
        else {
            warne(""+name+" ist kein gltiger Gegenstand.",this);
        }
        return null;
    }

    /**
     * Gibt dem Figur eine bestimmte Anzahl von Gegenstaenden einer Art
     * @param name Art des Gegenstands ("Koerner", "Tomate", "Giesskanne", "Ei", "Blume", "Sense")
     * @anzahl gewuenschte Anzahl
     */
    public void addAnzahlVonGegenstand(String name, int anzahl) {
        if (Gegenstand.istGegenstand(name)) { 
            for (int i=0; i<anzahl; i++) {
                inventar.add(Gegenstand.erzeuge(name));
            }
            zeichneImage();
        }
        else {
            warne(""+name+" ist kein gltiger Gegenstand.",this);
        }
    }

    /**
     * Liefert genau dann true, wenn der aufgerufene Figur keinen Gegenstand bei sich hat.
     * @return true, wenn der Figur keinen Gegenstand mit sich traegt, sonst false
     */
    public boolean istInventarLeer() {
        return inventar.size() == 0;
    }

    /**
     * Der Figur legt einen Gegenstand aus seinem Vorrat auf dem Feld ab.
     * @param name Einer der Gegenstaende ("Ei","Koerner","Tomate", "Giesskanne" oder "Sense")
     */      
    public void ablegen(String name)
    {
        if(getWorld() == null) return;

        if (!Gegenstand.istGegenstand(name)) { 
            warne(""+name+" ist kein gltiger Gegenstand.",this);
            return;
        } 
        if (getAnzahl(name)>0) {
            if(verbraucheEnergie(0,kostenWarten)) {
                int aktuellesX = this.getX();
                int aktuellesY = this.getY();
                for(Gegenstand g: inventar) {
                    if (g.getName().toUpperCase().equals(name.toUpperCase())) {
                        inventar.remove(g);
                        if(name.toUpperCase().equals("KOERNER") && istAuf("Acker") && !istAuf("Korn")) {
                            this.getWorld().addObject(Gegenstand.erzeuge("Korn"), aktuellesX, aktuellesY);
                        } else {
                            this.getWorld().addObject(g, aktuellesX, aktuellesY);
                        }

                        break;
                    }
                }
                refresh();
            }
        }
        else warne("Ich habe kein(e) "+name+" zum Ablegen!",this); 
    }

    /**
     * Der Figur nimmt einen Gegenstand, der sich auf seinem Feld befindet in seinen Vorrat auf.
     * Falls er nicht auf einem Gegenstand steht, gibt er eine Warnung aus.
     */      
    public void aufnehmen ()
    {
        if(getWorld() == null) return;

        if (getAnzahl()<=9900) 
        {
            List<Gegenstand> l = getWorld().getObjectsAt(getX(), getY(), Gegenstand.class);
            for(Gegenstand g : l)
            {
                if (g.istBeweglich()) {
                    if(verbraucheEnergie(0,kostenWarten)){
                        this.getWorld().removeObject(g);
                        inventar.add(g);
                        refresh();
                    }
                    return;
                } 
            }
            if(l.size()==1) {
                warne("Ein(e) "+l.get(0).getName()+" kann man nicht aufheben", this);
            }
            else
            {
                if(l.size()>1) {
                    warne("Nichts von dem was hier liegt, kann man aufheben", this);
                } else {
                    warne("Hier ist nichts zum Aufheben!",this);
                }
            }
        }
        else warne("Ich habe keinen Platz fr weitere Gegenstnde!",this); 
    }

    /**
     * Benutzt den angegebenen Gegenstand. Es kann nur die Giesskanne benutzt werden.
     * Steht man vor Wasser, wird sie gefllt. Steht man auf einem Ackerfeld
     * mit Tomate kann man diese gieen.
     * @param name Art des zu benutzenden Gegenstands ("Giesskanne")
     */
    public void benutze(String name) {
        if (!Gegenstand.istGegenstand(name) ) { 
            warne(""+name+" ist keine gltige Bezeichnung.",this);
            return;
        } 
        if(getAnzahl(name)== 0) {
            warne("Ich habe kein(e) "+name, this);
            return;
        }
        if(name.toUpperCase().equals("GIESSKANNE")) {
            Gegenstand g = getGegenstandAusInventar("Giesskanne");
            if(istVorne("Wasser")) {
                g.setzeBild(3);
            }
            if(istAuf("Acker")) {
                Tomate t = (Tomate) gibGegenstandUnterFigur("Tomate");
                if(g.getBild() > 0) {
                    g.setzeBild(g.getBild()-1);
                    if(t != null) {
                        t.giesse();
                    }
                }

            }
            refresh();
        }
    }


    /**
     * Diese Methode erntet reife Kornfelder oder Tomate
     * Der Ertrag wird im Inventar gespeichert.
     * @param name muss "Korn" oder "Tomate" sein
     */
    public void ernte(String name) {
        if(getWorld() == null) return;
        if(name.toUpperCase().equals("KORN")){
            if(istAuf("Korn")) {
                Gegenstand g = gibGegenstandUnterFigur("Korn");
                if(g.getWert() == 45) {
                    if(getAnzahl("Sense")>0) {
                        if(verbraucheEnergie(0, kostenWarten)){
                            addAnzahlVonGegenstand("Koerner", 5);
                            getWorld().removeObject(g);
                        }
                    } else warne("Ohne Sense kann ich kein Korn ernten.", this);
                } else warne("Das Korn ist noch nicht reif.", this);
            } else warne("Ich stehe gar nicht auf einem Kornfeld.", this);
            return;
        }
        if(name.toUpperCase().equals("TOMATE")){
            if(istAuf("Tomate")) {
                Gegenstand g = gibGegenstandUnterFigur("Tomate");
                if(g.getWert() == 100) {
                    if(verbraucheEnergie(0, kostenWarten)){
                        addAnzahlVonGegenstand("Tomate", 1);
                        ((Tomate) g).ernte();
                    }
                } else warne("Die Tomate ist noch nicht reif.", this);
            } else warne("Ich stehe gar nicht bei einer Tomate.", this);
            return;
        }        
        warne("Ich kann kein(e) "+name+" ernten.", this);
    }

    /**
     * Der Bauer macht Vesperpause. Dazu muss er direkt vor dem Eingang
     * des Bauernhauses stehen. Er betritt dieses, isst und trinkt (Energie
     * ist dann wieder 100) und steht dann in der Tr mit 
     * Blickrichtung nach unten.
     */
    public void vesperpause() {
        if(getWorld() == null) { error(); return; }
        List<Bauernhof> l = getWorld().getObjectsAt(xVor(), yVor(), Bauernhof.class);
        World w = getWorld();
        if(l.size() > 0 && l.get(0).istBetretbar(xVor(), yVor())) {
            durst = 100;
            hunger = 100;
            einsVor();
            warte();
            warte();
            warte();
            l.get(0).setzeBild(2);
            w.addObject(this, getX(), getY());
            setRotation(90);
            Greenfoot.delay(1);
        }
    }

    /** 
     * Hilfsmethode: Bitte nicht aufrufen
     */
    public boolean istBetretbar(int x, int y) {
        return !(x == getX() && y == getY());
    }

    /**
     * pruefe untersucht den mit "name" bezeichneten Gegenstand
     * Dieser muss entweder am gleichen Ort sein oder sich im Inventar 
     * befinden. 
     * @param name Bezeichnung des Gegenstands: "Gras", "Korn","Tomate", "Giesskanne" oder "Blume"
     * @return Wert zwischen 0-100 oder -1, je nach Zustand des Gegenstands
     */
    public int pruefe(String name) {
        if(getWorld() == null) { error(); return -1;}
        if(verbraucheEnergie(0, kostenWarten)){
            Gegenstand g = gibGegenstandUnterFigur(name);
            if(g == null && name.toUpperCase().equals("GIESSKANNE")) {
                g = getGegenstandAusInventar(name);
            }
            if(g == null) {
                warne(name +" ist hier gar nicht. Daher kann ich es nicht untersuchen.", this);
                return -1;
            }
            if(g instanceof Tomate && g.getWert() < 100 && ((Tomate) g).getDurst() == 0) {
                return -1;
            }
            return g.getWert();
        } return -1;
    }

    /**
     * Hilfsmethode: Bitte nicht aufrufen
     * Eine private Methode, die dafuer sorgt, 
     * dass der Bauer richtig gezeichnet wird.
     */
    protected void zeichneImage()
    {
        this.setImage(new GreenfootImage(originalImages.get(imageNumbers.get(bildnummer+getRotation()/90))));

        zeichneEnergie();
        zeichneInventar();
        this.getImage().rotate(-getRotation());
    }

    /**
     * Hilfsmethode: Bitte nicht aufrufen
     */
    private void zeichneInventar()
    {
        Gegenstand g = getGegenstandAusInventar("Giesskanne");
        if(g != null) {
            GreenfootImage inv = new GreenfootImage(g.getImage());
            inv.scale(35,35);
            this.getImage().drawImage(inv, 110, 110);
        }
        g = getGegenstandAusInventar("Sense"); 
        if(g != null) { 
            GreenfootImage inv = new GreenfootImage(g.getImage());
            inv.scale(40,80);
            this.getImage().drawImage(inv,110, 90);
        }
    }
}
