import greenfoot.*; // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.*;
import javax.swing.JOptionPane;
import java.io.File;  

/**
 * Diese Klasse ist die Oberklasse fuer die Figuren "Bauer", "Huhn" und "Schaf". 
 * Programme sollen nur in den Unterklassen 
 * implementiert werden, da diese Klasse fuer Java-Beginner sehr komplex ist.
 * Ein Figur kann sich felderweise in die vier Himmelsrichtungen im Szenario bewegen. 
 * Ueber Sensoren kann er Informationen ueber seine Umwelt erhalten.
 * @author T. Schaller 
 * @version 11.10.2023
 */

public class Figur extends MyActor
{
    /** gibt an, wie hungrig die Figur ist (0 = verhungert, 100 = satt) */
    protected int hunger;   
    /** gibt an, wie durstig die Figur ist (0 = verdurstet, 100 = kein Durst) */
    protected int durst;
    /** gibt an, wie viel Lebensenergie (Hunger+Durst) die Figur bei einer Bewegung verliert. */
    protected int kostenBewegung = 1;
    /** gibt an, wie viel Lebensenergie (Hunger+Durst) die Figur beim Warten verliert. */
    protected int kostenWarten = 1;
    /** gibt an, ob die Figur Hauptperson ist und nur durch den Benutzer gesteuert wird.
        andere Figuren agieren bei jeder Aktion der Hauptfigur (z.B. das Gras wchst) */
    public    boolean istHauptperson = true;
    private   int fehler = 0;
    private   Bauernhofprojekt world;

    /**
     * Ein Objekt der Klasse Figur wird instanziert.
     */
    public Figur(String name)
    {
        super(name, false, false);
        hunger=100;
        durst =100;
        betretbar = true;
        zeichneImage();
    }

    /** Hilfsmethode: bitte nicht aufrufen
     * sorgt dafr, dass das Programm abstrzt, wenn 
     * die Figur nicht in der Welt und Aktionen ausfhren soll
     */
    public void error() 
    {
        fehler++;
        if(fehler >= 20) { // Actor nicht in der Welt und Endlosschleife
            Figur f = null;
            f.einsVor();  // Gezielt zum Absturz bringen
            /*#
             * Wenn du hier landest, kam eine rote Fehlermeldung.
             * Du hast also einen Fehler in deinem Programm. 
             * Bitte ndere hier nichts, sondern bearbeite dein Programm. 
             * Schliee diesen Quelltext einfach.
             */
        }
    }

    /**
     * Gibt den Sttigungsgrad der Figur zurck
     * @return 0= gro - 100=satt
     */
    public int getHunger() {
        return hunger;
    }

    /**
     * Gibt die Durstigkeit der Figur zurck
     * @return 0= gro - 100=keinen Durst
     */
    public int getDurst() {
        return durst;
    }

    /**
     * Erhhrt den Hunger der Figur um den angegebenen Wert.
     * @param verlust Anzahl der verbrauchten Energiepunkte
     */
    public void verbraucheEnergie(int verlust) {
        hunger = hunger-verlust;
        if (hunger < 0) hunger=0;
        zeichneImage();
    }

    /**
     * Erhhrt den Hunger und Durst der Figur um den angegebenen Wert.
     * @param hunger Anzahl der verbrauchten Energiepunkte (Hunger)
     * @param durst  Anzahl der verbrauchten Energiepunkte (Durst)
     * @return gibt zurck, ob es mglich war, so viele Punkte abzuziehen
     */
    protected boolean verbraucheEnergie(int hunger, int durst) {
        if(this.hunger<hunger && istHauptperson) {
            warne("Ich habe zu viel Hunger!",this);
            return false;
        }
        if(this.durst<durst && istHauptperson) {
            warne("Ich habe zu viel Durst!", this);
            return false;
        }
        this.hunger-=hunger;
        this.durst -=durst;
        if(this.hunger < 0 ) this.hunger = 0;
        if(this.durst < 0) this.durst = 0;
        zeichneImage();
        return true;
    }

    /**
     * Gibt die x-Koordinate zurck, wenn der Figur einen Schritt nach vorne macht.
     * @return x-Koordinate
     */
    protected int xVor() {
        if(this.getWorld()==null) { error(); return -1; }
        switch(this.getRotation())
        {
            case 0 : return getX()+1; 
            case 180 : return getX()-1; 
        }
        return getX();
    }

    /**
     * Gibt die y-Koordinate zurck, wenn die Figur einen Schritt nach vorne macht.
     * @return y-Koordinate
     */

    protected int yVor() {
        if(this.getWorld()==null) { error(); return -1;}
        switch(this.getRotation())
        {
            case 90 : return getY()+1;  
            case 270 : return getY()-1; 
        }
        return getY();
    }

    /**
     * die Figur geht einen Schritt in die Blickrichtung.
     * Das macht er nur, wenn vor ihm nicht im Weg ist.
     */
    public void einsVor()
    {
        if(getWorld() == null) { error(); return;}

        if (!this.istVorneFrei()) {
            warne("Ich kann keinen Schritt machen!\nEin Hindernis ist im Weg!",this); 
            return;
        }

        if (verbraucheEnergie(kostenBewegung,kostenWarten))
        {   
            this.setLocation(xVor(), yVor());
            refresh();
        }

    }

    /**
     * die Figur dreht sich um 90 Grad nach rechts (aus der Sicht des Figurs).
     */
    public void dreheRechts()
    {
        if(getWorld() == null) { error(); return;}
        if (verbraucheEnergie(kostenBewegung,kostenWarten)) {
            setRotation((getRotation()+90)%360);
            refresh();
        } 
    }

    /**
     * die Figur dreht sich um 90 Grad nach links (aus der Sicht des Figurs).
     */
    public void dreheLinks()
    {
        if(getWorld() == null) { error(); return;}
        if(verbraucheEnergie(kostenBewegung,kostenWarten)){
            setRotation((getRotation()+270)%360);
            refresh();
        }
    }

    /**
     * Die Figur wartet und macht nichts. In der Zeit kann aber z.B.
     * das Gras wachsen.
     */
    public void warte() {
        if(verbraucheEnergie(0,kostenWarten)) {
            refresh();
        }
    }

    /** 
     * Hilfsmethode, bitte nicht benutzen 
     */
    protected void refresh() {
        zeichneImage();
        if(istHauptperson) {
            world.fuehreAktionenDurch();
            Greenfoot.delay(1);
        }
    }

    /**
     * pruefe untersucht den mit "name" bezeichneten Gegenstand
     * Dieser muss entweder am gleichen Ort sein. 
     * @param name Bezeichnung des Gegenstands: "Gras", "Korn","Tomate", "Giesskanne" oder "Blume"
     * @return Wert zwischen 0-100 oder -1, je nach Zustand des Gegenstands
     */
    public int pruefe(String name) {
        if(getWorld() == null) { error(); return -1;}
        if(verbraucheEnergie(0, kostenWarten)){
            Gegenstand g = gibGegenstandUnterFigur(name);

            if(g == null) {
                warne(name +" ist hier gar nicht. Daher kann ich es nicht untersuchen.", this);
                return -1;
            }
            if(g instanceof Tomate && g.getWert() < 100 && ((Tomate) g).getDurst() == 0) {
                return -1;
            }
            return g.getWert();
        } return -1;
    }

    /**
     * Liefert genau dann true, wenn auf der Kachel, auf der sich der
     * aufgerufene Figur gerade befindet, ein beliebiger Gegenstand befindet.
     * @return true, wenn die Figur auf einem Gegenstand steht, sonst false
     */
    public boolean istAufGegenstand() {
        if(getWorld() == null) { error(); return false;}

        List g = getWorld().getObjectsAt(getX(), getY(), Gegenstand.class);
        if (g.size() > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * Liefert genau dann true, wenn auf der Kachel, auf der sich der
     * aufgerufene Figur gerade befindet, ein Gegenstand mit dem angegebnen Namen befindet.
     * @param name Einer der Gegenstaende ("Gras","Nest","Ei","Koerner","Acker","Korn","Tomate", "Giesskanne", "Blume", "Sense")
     * @return true, wenn die Figur auf einem Gegenstand steht, sonst false
     */
    public boolean istAuf(String name) {
        if (!Gegenstand.istGegenstand(name)) { 
            melde(""+name+" ist kein gltiger Gegenstand.",true);
            return false;
        } 
        if(getWorld() == null) { error(); return false;}

        List<MyActor> g = getWorld().getObjectsAt(getX(), getY(), MyActor.class);
        for(MyActor a : g) {
            if(a.getName().toUpperCase().equals(name.toUpperCase())) return true;
        } 
        return false;

    }

    /**
     * Gibt den Gegenstand zurueck, auf dem sich ein Figur befindet 
     * Wenn dort kein Gegenstand ist, dann wird null zurckgegeben
     * @return Gegenstand (Objekt vom Typ "Gras","Nest","Ei","Koerner","Acker","Korn","Tomate", "Giesskanne", "Blume" oder "Sense")
     */
    protected Gegenstand gibGegenstandUnterFigur() {
        if(getWorld() == null) { error(); return null;}

        List<Gegenstand> g = getWorld().getObjectsAt(getX(), getY(), Gegenstand.class);
        if(g.size()>0) {
            return g.get(0);
        } 
        return null;
    }

    /**
     * Gibt den Gegenstand vom gesuchten Typ zurueck, wenn sich die Figur darauf befindet 
     * @param name Bezeichnung des Gegenstandes ("Gras","Nest","Ei","Koerner","Acker","Korn","Tomate", "Giesskanne", "Blume", "Sense")
     * @return Gegenstand 
     */
    protected Gegenstand gibGegenstandUnterFigur(String name) {
        if(getWorld() == null) { error(); return null;}

        List<Gegenstand> g = getWorld().getObjectsAt(getX(), getY(), Gegenstand.class);
        for(Gegenstand a : g) {
            if(a.getName().toUpperCase().equals(name.toUpperCase())) return a;
        } 
        return null;
    }

    /**
     * Liefert genau dann true, wenn auf der Kachel vor der Figur
     * in Blickrichtung ein Gegenstand mit dem angegebnen Namen befindet.
     * @param name Einer der Gegenstaende ("Gras","Zaun","Wasser","Nest","Ei","Koerner","Acker","Korn","Stall","Tomate", "Giesskanne","Bauernhof", "Blume", "Sense")
     * @return true, wenn die Figur vor diesem Gegenstand steht, sonst false
     */
    public boolean istVorne(String name) {
        if (!Gegenstand.istGegenstand(name)) { 
            melde(""+name+" ist kein gltiger Gegenstand.",true);
            return false;
        } 
        if(getWorld() == null) { error(); return false;}

        List<MyActor> g = getWorld().getObjectsAt(xVor(), yVor(), MyActor.class);
        for(MyActor a : g) {
            if(a.getName().toUpperCase().equals(name.toUpperCase())) return true;
        } 
        return false;
    }

   /**
     * Gibt den Gegenstand vom gesuchten Typ zurueck, wenn sich die Figur davor befindet 
     * @param name Bezeichnung des Gegenstandes ("Gras","Nest","Ei","Koerner","Acker","Korn","Tomate", "Giesskanne", "Blume", "Sense")
     * @return Gegenstand 
     */
    protected Gegenstand gibGegenstandVorFigur(String name) {
        if(getWorld() == null) { error(); return null;}

        List<Gegenstand> g = getWorld().getObjectsAt(xVor(), yVor(), Gegenstand.class);
        for(Gegenstand a : g) {
            if(a.getName().toUpperCase().equals(name.toUpperCase())) return a;
        } 
        return null;
    }

    /**
     * Die Methode ueberprueft, ob vor der Figur Hindernisse stehen oder nicht
     * @return true, wenn vor der Figur frei ist, sonst false
     */     
    public boolean istVorneFrei()
    {
        if(getWorld() == null) { error(); return false;}
        List<MyActor> liste = new ArrayList<MyActor>();
        liste.addAll(getWorld().getObjectsAt(xVor(), yVor(), Gegenstand.class));
        if(!betretbar)   // Hhner knnen betreten werden und knnen auch zu anderen Figuren aufs Feld.
            liste.addAll(getWorld().getObjectsAt(xVor(), yVor(), Figur.class));

        for(MyActor a : liste) {
            if(!a.istBetretbar(xVor(), yVor())) return false;
        } 

        return true;
    }

    /**
     * Die Methode ueberprueft, ob rechts der Laufrichtung frei ist.
     * @return true, wenn das Feld rechts betreten werden kann, sonst false
     */    
    public boolean istRechtsFrei()
    {
        if(getWorld() == null) { error(); return false;}

        int lRichtung = getRotation();

        int neuX = getX();
        int neuY = getY();
        if (lRichtung == 0)  neuY++;  
        if (lRichtung == 90)  neuX--;
        if (lRichtung == 180)  neuY--;
        if (lRichtung == 270)  neuX++;
        List<MyActor> liste = new ArrayList<MyActor>();
        liste.addAll(getWorld().getObjectsAt(neuX, neuY,Gegenstand.class));
        if(!betretbar)   // Hhner knnen betreten werden und knnen auch zu anderen Figuren aufs Feld.
            liste.addAll(getWorld().getObjectsAt(neuX, neuY, Figur.class));

        for(MyActor a : liste) {
            if(!a.istBetretbar(neuX, neuY)) return false;
        } 
        return true;
    }

    /**
     * Die Methode ueberprueft, ob sich links der Laufrichtung eine Wand befindet.
     * @return true, wenn das Feld links betreten werden kann, sonst false
     */  
    public boolean istLinksFrei()
    {
        if(getWorld() == null) { error(); return false;}

        int lRichtung = getRotation();

        List<MyActor> g=null;

        int neuX = getX();
        int neuY = getY();
        if (lRichtung == 0)  neuY--;  
        if (lRichtung == 90)  neuX++;
        if (lRichtung == 180)  neuY++;
        if (lRichtung == 270)  neuX--;
        List<MyActor> liste = new ArrayList<MyActor>();
        liste.addAll(getWorld().getObjectsAt(neuX, neuY,Gegenstand.class));
        if(!betretbar)   // Hhner knnen betreten werden und knnen auch zu anderen Figuren aufs Feld.
            liste.addAll(getWorld().getObjectsAt(neuX, neuY, Figur.class));

        for(MyActor a : liste) {
            if(!a.istBetretbar(neuX, neuY)) return false;
        } 
        return true;
    }

    /**
     * Hilfsmethode, die dafuer sorgt, dass der "Aufdruck" auf der Figur (Anzahl der Gegenstaende), der Akkustandsbalken und
     * der Richtungspfeil aktualisiert werden
     * Bitte nicht aufrufen.
     */
    protected void zeichneImage()
    {
        this.setImage(new GreenfootImage(originalImages.get(imageNumbers.get(bildnummer+getRotation()/90))));

        zeichneEnergie();

        this.getImage().rotate(-getRotation());
    }

    /**
     * Hilfsmethode: Zeichne / Aktualisiere die hunger (Akkustandsbalken), die die Figur hat
     * Bitte nicht aufrufen
     */
    protected void zeichneEnergie()
    {
        final int breite = 50;
        final int hoehe = 3;
        final int x=(getImage().getWidth()-breite)/2;
        final int y=0;
        this.getImage().setColor(Color.GREEN);
        this.getImage().fillRect(x, y, breite, hoehe);
        int breiteNeu = breite-(int)(Math.floor(breite*(hunger/100.0)));
        this.getImage().setColor(Color.RED);
        this.getImage().fillRect(x+breite-breiteNeu, y, breiteNeu, hoehe);
        this.getImage().setColor(Color.BLACK);
        this.getImage().drawRect(x, y, breite, hoehe);
        this.getImage().drawRect(x, y, breite/2, hoehe);

        this.getImage().setColor(Color.BLUE);
        this.getImage().fillRect(x, y+hoehe, breite, hoehe);
        breiteNeu = breite-(int)(Math.floor(breite*(durst/100.0)));
        this.getImage().setColor(Color.RED);
        this.getImage().fillRect(x+breite-breiteNeu, y+hoehe, breiteNeu, hoehe);
        this.getImage().setColor(Color.BLACK);
        this.getImage().drawRect(x, y+hoehe, breite, hoehe);
        this.getImage().drawRect(x, y+hoehe, breite/2, hoehe);
    }

    // Systembedingte Routinen    
    /* setLocation(x,y) und setRotation(x) koennen nicht verborgen werden, 
     * weil sie schon in der Oberklasse Actor public sind. 
     */

    /**
     * Hilfsmethode: bitte nicht aufrufen.
     * setLocation(x,y) fuer Figur wird ueberschrieben, um nicht auf Wand oder
     * anderen Figur ziehen zu koennen!
     * -- setzt den Figur auf eine Kachel, deren Spalte x sowie Reihe y du 
     * hier vorgibst. Die Nummerierung beginnt jeweils bei 0 !!) 
     * @param x die x-Koordinate
     * @param y die y-Koordinate
     */
    public void setLocation(int x, int y) {
        if(getWorld() == null) { error(); return;}
        if( (!((x==getX() && y==getY()) || (Math.abs(x-getX())+Math.abs(y-getY())<=1))) ) {   // Verhindern des Verschiebens des Figurs
            warne("Ich bin ein Figur. Ich kann nicht fliegen.",this);
        }
        else {
            List<MyActor> liste = new ArrayList<MyActor>();
            liste.addAll(getWorld().getObjectsAt(x, y,Gegenstand.class));
            if(!betretbar)   // Hhner knnen betreten werden und knnen auch zu anderen Figuren aufs Feld.
                liste.addAll(getWorld().getObjectsAt(x, y, Figur.class));

            liste.remove(this);
            boolean frei = true;
            for(MyActor a : liste) {
                if(!a.istBetretbar(x, y)) frei = false;
            } 
            if (frei) {
                super.setLocation(x, y);
            }
            else warne ("Ich kann nicht an diesen Ort. Da ist kein Platz fuer mich!",this);
        }
    }

    /**
     * Hilfsmethode: Bitte nicht aufrufen
     * setRotation(x) fuer Figur wird ueberschrieben, damit nur vier Rotationsrichtungen moeglich sind!
     * @param x die Richtung (0=Osten, 90=Sueden, 180=Westen, 270=Norden)
     */
    public void setRotation(int x) {
        if(getWorld() == null) { error(); return;}
        if (x==0 || x==90 || x==180 || x==270) super.setRotation(x);
        this.zeichneImage();
    }

    /**
     * Hilfsmethode: bitte nicht aufrufen.
     */
    public void addedToWorld(World w) {
        super.addedToWorld(w);
        world = (Bauernhofprojekt) w;
    }    
}