import greenfoot.*; // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.Random;
import java.util.List;
import javax.swing.JOptionPane;
import java.util.ArrayList;
import java.util.Iterator;
import javax.swing.JFrame;
import javax.swing.JFileChooser;
import java.io.File;
import java.nio.file.*;
import java.util.regex.Pattern;
import java.util.regex.Matcher;
import java.lang.reflect.*;

/**
 * Die einzigen aktiven Akteure in der Bauernhof sind die Figur.
 * Die Welt besteht aus 14 * 10 Feldern.
 * 
 * @author T. Schaller 
 * @version 04.10.2021
 */
public class Bauernhofprojekt extends World
{
    private static int zellenGroesse = 60;

    private ReadWrite secret = new ReadWrite();
    private GreenfootImage[] backgroundimages;
    private int[][] map;
    private ArrayList<Runnable> aktionen;

    private int time;
    private Message timeText;

    /**
     * Erschaffe eine Welt mit 14 * 12 Zellen.
     */
    public Bauernhofprojekt()
    {
        super(14, 12, zellenGroesse);

        aktionen = new ArrayList<Runnable>();

        backgroundimages = new GreenfootImage[2];
        backgroundimages[0] = new GreenfootImage("images/Bodenplatte.png");

        setBackground("images/Bodenplatte.png");

        setActOrder(Figur.class);
        setPaintOrder(Message.class, Figur.class, Gegenstand.class);

        time = 0;

        Greenfoot.setSpeed(25);
        if (secret.getLevel()==-1) secret.init(); // Falls fehlerhafte Secret-Datei

        if (secret.getLevel()==1) ab1_FigurSteuern();
        if (secret.getLevel()==2) ab2_FigurProgrammieren();
        if (secret.getLevel()==3) ab3_wiederholer();
        if (secret.getLevel()==4) ab4_verzweiger();
        if (secret.getLevel()==5) ab5_funktionen();
        if (secret.getLevel()==6) ab6_attribute();
        if (secret.getLevel()==7) ab7_lokale_variablen_zaehlschleifen();
        if (secret.getLevel()==8) ab8_arrays();
        if (secret.getLevel()==9) ab9_finalBild();
    }

    /**
     * Das Figurszenario auf die Anfangswerte (Level 1) zurcksetzen
     */
    public void neuStart() {
        secret.init();
        ab1_FigurSteuern();
    }

    /**
     * In ein niedrigeres Level wechseln
     * @param int   Nummer des Levels, in das gewechselt werden soll
     */
    public void geheZuLevel (int i) {

        if (i>0 && i<=secret.getMaxLevel()) {
            secret.writeLevel(i,true);
            Greenfoot.setWorld(new Bauernhofprojekt());
        } else {
            melde("Du hast erst Level "+secret.getMaxLevel()+ " erreicht. ");
        }
    }

    /** 
     * Secret-Datei laden, um ein hheres Level freizuschalten
     * 
     */
    private void ladeSecretDatei() {
        int oldLevel = secret.getLevel();
        try{ 
            JFileChooser fileChooser = new JFileChooser(".");
            fileChooser.setDialogTitle("Secret-Datei laden");
            fileChooser.setApproveButtonText("Level-Datei laden");
            JFrame fcframe = new JFrame();
            fcframe.toFront();
            fcframe.setAlwaysOnTop(true);

            if(fileChooser.showOpenDialog(fcframe)==JFileChooser.APPROVE_OPTION){
                File file = fileChooser.getSelectedFile();
                if (file != null) {
                    Path FROM = file.toPath();
                    Path TO   = Paths.get(".\\secret");
                    //overwrite existing file, if exists
                    CopyOption[] options = new CopyOption[]{
                            StandardCopyOption.REPLACE_EXISTING,
                            StandardCopyOption.COPY_ATTRIBUTES
                        };
                    Files.copy(FROM, TO, options);
                    int max = secret.getMaxLevel();
                    int level = secret.getLevel();
                    if(max >= 1 && max <= 11 && level>=1 && level <=11) {
                        melde("Level 1-"+max+" sind nun freigeschaltet. Gehe zu Level "+level+".");
                    } else {
                        throw new IllegalArgumentException( "Keine richtige Secret-Datei." ); 
                    }
                    Greenfoot.setWorld(new Bauernhofprojekt());
                }
            }
        }
        catch(Exception e) {
            melde("Fehlerhafte Secret-Datei. Bleibe in Level "+oldLevel+".");
            secret.init();
            secret.writeLevel(oldLevel,true);
            Greenfoot.setWorld(new Bauernhofprojekt());
        }

    }

    /**
     * Die Welt wird geleert
     */
    private void weltLeeren()
    {
        removeObjects(getObjects(null));
        Greenfoot.delay(1);
        aktionen.clear();
    }

    public void fuehreAktionenDurch() {
        time++;
        timeText.changeText("Zeit: "+time);
        Greenfoot.delay(0);

        for(int i = 0; i < aktionen.size(); i++) {
            aktionen.get(i).run();
        }
    }

    public void addAktion(Runnable r) {
        aktionen.add(r);
    }

    /** 
     * Liefert die Information, ob es sich um eine Welt mit Schwerkraft handelt
     */

    /** 
     * berschrift oben links ausgeben.
     */
    private void zeichneUeberschrift(String text) {
        this.getBackground().setColor(Color.LIGHT_GRAY);
        this.getBackground().fillRect(0,  0,300,20);
        this.getBackground().setColor(Color.BLACK);
        this.getBackground().setFont(new Font("Verdana",true,false,12));
        this.getBackground().drawString(text, 2, 14);
    }

    private Actor generateActor(int nr) {
        if(nr >= 10){
            Gegenstand a = Gegenstand.erzeuge(nr/100-1);
            if(a != null) {
                a.setzeBild(nr%100);
                if(a instanceof Runnable) addAktion((Runnable) a);
            }
            return a;
        } 
        else  return null;

    }

    /**
     * Generiert eine Welt aus dem angegeben 2-dim. Array
     * Dabei werden sowohl die Hintergrundbilder gesetzt, als auch Objekte
     * in die Welt eingefgt.
     * @param backg 2-dim. Array, das die Welt beschreibt.
     */
    private void generateWorld(int[][] backg)
    {
        weltLeeren();
        map = backg;
        GreenfootImage back = new GreenfootImage(840,720);
        for(int x=0; x<14; x++) {
            for(int y=0; y<12; y++) {
                back.drawImage(backgroundimages[0], x*zellenGroesse, y*zellenGroesse);
                Actor a = generateActor(backg[y][x]);
                if (a != null) addObject(a,x,y);
            }
        }
        this.setBackground(back);

        time = 0;
        timeText = new Message("Zeit: "+time, true);
        addObject(timeText,0,0);

    }

    /**
     * Zeigt die Koordinaten aller Felder an
     */
    public void zeigeKoordinaten(){
        int delta;
        for (int x=0; x<getWidth(); x++){
            for (int y=0; y<getHeight(); y++){
                this.getBackground().setColor(Color.BLACK);
                this.getBackground().setFont(new Font("Verdana",false,false,12));
                if (x>9 || y>9) {
                    delta = 4;
                    if (x>9 && y>9) delta = 8;
                }
                else delta = 0;
                this.getBackground().drawString("("+x+"|"+y+")", x*60+14-delta, y*60+35);
            }
        }
    }

    /**
     * Erzeugt die Welt neu, damit die Koordinaten wieder verschwinden
     */
    public void versteckeKoordinaten(){
        Greenfoot.setWorld(new Bauernhofprojekt());
    }

    /**
     * Testroutinen
     */
    private int pos_x;
    private int pos_y;
    private int rot;
    private void savePosition(Actor a) {
        pos_x = a.getX();
        pos_y = a.getY();
        rot = a.getRotation();
    }

    private boolean checkPosition(Actor a) {
        return checkPosition(a, 0 ,0 , 0);
    }

    private boolean checkPosition(Actor a, int dRotation) {
        return checkPosition(a, dRotation, 0, 0);
    }

    private boolean checkPosition(Actor a, int dx, int dy) {
        return checkPosition(a, 0, dx, dy);
    }

    private boolean checkPosition(Actor a, int dRotation, int dx, int dy) {
        if(pos_x+dx != a.getX() || pos_y+dy != a.getY()) {
            melde("FEHLER: Die Position stimmt nicht!");
            return false;
        }
        if((rot +dRotation+360)%360 != a.getRotation()) {
            melde("FEHLER: Die Figur schaut in die falsche Richtung.");
            return false;
        }
        return true;
    }

    private Method getMethod(Class c, String name, int nr){
        Method m[] = c.getDeclaredMethods();
        int i=0;
        for (i = 0; i < m.length; i++){
            if(m[i].getName().toLowerCase().contains(name.toLowerCase())) break;
        }
        if(i == m.length) {
            melde("FEHLER: Ich finde keine Methode, die '"+name+"' im Namen trgt.\nBitte benenne deine Methode fr Aufgabe "+nr+" entsprechend.");
            return null;
        } else {
            return m[i];
        }
    }

    /* Es folgen nun die Definitionen der Level und Einstze                      */
    /* ---------------------------------------------------------------------------*/

    private void ab1_FigurSteuern()
    {
        int[][] backg = {{  0,  0,  0,  0, 0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,905,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {203,200,200,902,903,904,200,200,200,200,200,200,200,204},
                {205,  0,  0,901,  0,900,  0,  0,308,307,307,307,306,205},
                {205,104,  0,  0,  0,  0,104,  0,309,310,310,310,304,205},
                {205,104,104,  0,104,  0,  0,  0,302,305,303,310,304,205},
                {205,  0,  0,  0,104,  0,104,  0,104,  0,302,305,301,205},
                {205,104,104,  0,104,104,104,104,  0,104,  0,  0,104,205},
                {205,104,  0,  0,  0,  0,  0,104,  0,  0,  0,104,104,205},
                {205,  0,104,104,  0,104,  0,  0,104,  0,  0,104,  0,205},
                {205,  0,  0,  0,  0,  0,104,  0,  0,  0,104,  0,104,205},
                {201,200,200,200,200,200,200,200,200,200,200,200,200,202}};

        weltLeeren();
        generateWorld(backg);
        zeichneUeberschrift("AB1 - Figur von Hand steuern");

        AB1_Schaf starter = new AB1_Schaf();
        addObject(starter, 2, 6);
        starter.setRotation(180);
        starter.refresh();
        AB1_Schaf starter1 = new AB1_Schaf();
        addObject(starter1, 7, 8);
        starter1.verbraucheEnergie(55,30);
        starter1.refresh();
        AB1_Schaf starter3 = new AB1_Schaf();
        addObject(starter3,12, 9);
        starter3.setRotation(270);
        starter3.verbraucheEnergie(80,70);
        starter3.refresh();
    }

    public void checkup_01() {
        List<Schaf> schafe = getObjects(Schaf.class);
        boolean ok = true;
        for(Schaf s : schafe) {
            if(s.getHunger() < 50 || s.getDurst() < 50) ok = false;
        }
        if (ok) {
            secret.maxLevel(2);
            melde("Du hast die Schafe gut gesteuert. Alle sind zufrieden.\nDu kannst nun in Level 2 wechseln (Methode geheZuLevel).");
        } else {
            melde("Du hast die Aufgabe noch nicht erfllt");
        }

    }

    private void ab2_FigurProgrammieren()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {203,200,200,200,200,200,200,200,200,200,200,200,200,204},
                {205,104,  0,  0,  0,  0,104,  0,104,  0,  0,104,  0,205},
                {205,  0,  0,104,  0,  0,  0,  0,104,308,307,307,306,205},
                {205,104,  0,  0,300,  0,104,  0,104,309,310,310,304,205},
                {205,104,104,  0,104,  0,  0,104,  0,302,305,305,301,205},
                {209,200,200,200,200,200,207,200,200,200,200,200,200,206},
                {205,  0,  0,  0,  0,  0,205,  0,  0,104,  0,  0,  0,205},
                {205,104,  0,  0,  0,  0,205,  0,102,102,102,102,  0,205},
                {205,  0,104,104,  0,  0,205,  0,104,  0,  0,  0,  0,205},
                {205,  0,  0,  0,  0,  0,205,  0,  0,  0,104,  0,104,205},
                {201,200,200,200,200,200,208,200,200,200,200,200,200,202}};

        weltLeeren();
        generateWorld(backg);
        zeichneUeberschrift("AB2 - Methoden erstellen");
        Schaf starter = new AB2_Schaf();
        addObject(starter, 3, 4);
        Schaf starter1 = new AB2_Schaf();
        addObject(starter1,1, 10);
        Schaf starter2 = new AB2_Schaf();
        addObject(starter2,8, 8);
    }

    public void checkup_02()
    {
        ab2_FigurProgrammieren();

        List<AB2_Schaf> schafe = this.getObjects(AB2_Schaf.class);
        melde("Teste dreheRunde() ...",1000);
        AB2_Schaf s = schafe.get(1);
        savePosition(s);
        s.dreheRunde();
        if(!checkPosition(s)) return;

        melde("Teste dreheRunde(): OK.",1500);

        melde("Teste dreheUm() ...",1000);
        for(AB2_Schaf s2 : schafe){
            savePosition(s2);
            s2.dreheUm();
            if(!checkPosition(s2,180)) return;
            s2.dreheUm();
            if(!checkPosition(s2)) return;
        }
        melde("Teste dreheUm(): OK.",1500);

        melde("Teste frissDichSatt() ...",1000);
        savePosition(schafe.get(2));
        int h = schafe.get(2).getHunger();
        schafe.get(2).fresseDichSatt();
        if(!checkPosition(schafe.get(2),3,0)) return;
        Gras g1 = this.getObjectsAt(pos_x, pos_y, Gras.class).get(0);
        Gras g2 = this.getObjectsAt(pos_x+1, pos_y, Gras.class).get(0);
        Gras g3 = this.getObjectsAt(pos_x+2, pos_y, Gras.class).get(0);
        Gras g4 = this.getObjectsAt(pos_x+3, pos_y, Gras.class).get(0);
        if(!(g1.getWert() ==3 || g4.getWert() == 0) || g3.getWert() != 1 || g2.getWert() !=2){
            melde("FEHLER: Es wurde nicht genau 3 Grasstcke gefressen.");
            return;
        }

        melde("Teste frissDichSatt(): OK.",1500);

        melde("Teste Haken schlagen...",1000);
        Class c = AB2_Schaf.class;
        Method m[] = c.getDeclaredMethods();
        int i=0;
        for (i = 0; i < m.length; i++){
            if(m[i].getName().toLowerCase().contains("haken")) break;
        }
        if(i == m.length) {
            melde("FEHLER: Ich finde keine Methode, die 'haken' im Namen trgt.");
            return;
        }

        savePosition(schafe.get(0));
        try{
            m[i].invoke(schafe.get(0));
            if(!checkPosition(schafe.get(0),2,0)) return;
            schafe.get(0).dreheUm();
            m[i].invoke(schafe.get(0));
            if(!checkPosition(schafe.get(0),180)) return;
        } catch(Throwable e) {
            melde("FEHLER: Die Methode wurde falsch deklariert.");
            return;
        }

        melde("Teste Haken schlagen: OK.",1500);

        melde("Teste dreheRunde Variante 2...",1000);
        for (i = 0; i < m.length; i++){
            if(m[i].getName().toLowerCase().contains("dreherunde") && !m[i].getName().toLowerCase().equals("dreherunde")) break;
        }
        if(i == m.length) {
            melde("FEHLER: Ich finde keine zweite Methode,\n die 'dreheRunde' im Namen trgt.");
            return;
        }

        savePosition(schafe.get(1));
        try{
            m[i].invoke(schafe.get(1));
        } catch(Throwable e) {
            melde("FEHLER: Die Methode wurde falsch deklariert.");
            return;
        }
        if(!checkPosition(schafe.get(1),0,0)) return;
        melde("Teste dreheRunde Variante 2: OK.",1500);
        melde("alle Tests bestanden\nLevel 3 freigeschaltet.");
        secret.maxLevel(3);
    }

    public void leveltest_02()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,1200, 0},
                {200,200,200,200,200,200,200,200,200,  0,  0,  0,  0,  0},
                {  0,  0,  0,104,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {  0,104,  0,  0,300,  0,104,103,103,103,103,103,103,103},
                {  0,104,104,  0,104,  0,  0,103,103,103,103,103,300,103},
                {203,200,200,200,200,200,204,103,103,103,103,103,103,103},
                {205,  0,  0,  0,  0,  0,205,103,103,103,103,103,103,103},
                {205,104,  0,  0,  0,  0,205,103,103,103,103,103,103,103},
                {205,  0,104,104,  0,  0,  0,  0,300,  0,  0,  0,  0,  0},
                {205,  0,  0,  0,  0,  0,205,  0,  0,  0,104,  0,104,  0},
                {201,200,200,200,200,200,202,  0,  0,  0,  0,  0,  0,  0}};

        weltLeeren();
        generateWorld(backg);
        zeichneUeberschrift("Leveltest 2: Heim in den Stall");
        AB2_Schaf ab2 = new AB2_Schaf();
        addObject(ab2,  1, 9);
        ab2.verbraucheEnergie(60,80);

        ab2.leveltest2();
        Greenfoot.delay(2);
        if (ab2.getWorld()==null && ab2.getX() == 9 && ab2.getY() == 3) {
            secret.writeLevel(3);
            melde("Super! Das Schaf ist wohlbehalten im Stall angekommen.");
        } else {
            melde("Du hast die Aufgabe noch nicht erfllt");
        }
        Greenfoot.setWorld(new Bauernhofprojekt());
    }

    private void ab3_wiederholer()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,1200,  0,  0,  0,  0},
                {203,200,200,200,200,200,  0,  0,  0,  0,  0,  0,  0,  0},
                {205,  0,  0,104,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {205,104,  0,  0,300,  0,104,  0,  0,104,  0,  0,  0,  0},
                {205,300,104,  0,104,  0,  0,  0,  0,104,  0,  0,  0,308},
                {209,200,200,204,  0,  0,  0,  0,  0,  0,  0,  0,  0,302}, 
                {205,103,103,209,200,200,200,204,  0,  0,104,  0,  0,  0},
                {205,103,103,205,104,104,104,209,200,200,200,200,200,204},
                {205,103,103,205,104,104,104,205,100,100,100,100,100,205},
                {205,103,103,205,104,104,104,205,100,100,100,100,100,205},
                {201,200,200,208,200,200,200,208,200,200,200,200,200,202}};

        weltLeeren();
        generateWorld(backg);
        zeichneUeberschrift("AB3 - Wiederholungen mit while-Schleifen");
        Random r = new Random();
        Schaf f;
        f = new AB3_Schaf();
        addObject(f, 1, 7);
        f.setRotation(90);
        f.verbraucheEnergie(r.nextInt(60)+10,r.nextInt(40));
        f = new AB3_Schaf();
        addObject(f, 4, 10);
        f.verbraucheEnergie(r.nextInt(60)+10,r.nextInt(40));
        f = new AB3_Schaf();
        addObject(f, 8, 10);
        f.verbraucheEnergie(r.nextInt(40)+10,r.nextInt(30));
        f = new AB3_Schaf();
        addObject(f, 12,  4);
        f.verbraucheEnergie(r.nextInt(30)+10,r.nextInt(70));
        f.setRotation(180);
        f = new AB3_Schaf();
        addObject(f, 6,  5);
        f.verbraucheEnergie(r.nextInt(60)+10,r.nextInt(70));
        f.setRotation(180);
        f = new AB3_Schaf();
        addObject(f, 11, 6);
        f.verbraucheEnergie(r.nextInt(40)+10,r.nextInt(30));

    }

    public void checkup_03()
    {
        ab3_wiederholer();

        List<AB3_Schaf> schafe = this.getObjects(AB3_Schaf.class);
        melde("Teste wieseErkunden()...",1000);
        for(int i=0; i<3;i++) {
            AB3_Schaf s = schafe.get(i);
            savePosition(s);
            s.wieseErkunden();
            if(!checkPosition(s)) return;
        }
        melde("Teste wieseErkunden(): OK.",1500);

        melde("Teste vorBisGrasUndFriss() ...",1000);
        for(int i=3; i<5;i++) {
            AB3_Schaf s = schafe.get(i);
            s.vorBisGrasUndFriss();
            List<Gras> g = getObjectsAt(s.getX(), s.getY(), Gras.class);
            if(g.size()!=1) {
                melde("FEHLER: Das Schaf ist nicht bis zum Gras gelaufen.");
                return;
            }
            if(g.get(0).getWert() != 0) {
                melde("FEHLER: Das Schaf hat das Gras nicht gefressen.");
                return;
            }
        }
        melde("Teste vorBisGrasUndFriss(): OK.",1500);

        Method m=null;
        try{
            melde("Teste vorBisWasserUndTrinke() ...",1000);
            m = getMethod(AB3_Schaf.class, "Wasser", 5);
            if(m == null) return;
            for(int i=3; i<6;i++) {
                AB3_Schaf s = schafe.get(i);
                m.invoke(s);

                if(!s.istVorne("Wasser")) {
                    melde("FEHLER: Das Schaf ist nicht bis zum Wasser gelaufen.");
                    return;
                }
                if(s.getDurst()!=100) {
                    melde("FEHLER: Das Schaf hat nicht genug Wasser getrunken.");
                    return;
                }

            }
            melde("Teste vorBisWasserUndTrinke(): OK.",1500);

            melde("Teste sattFressen() ...",1000);
            m = getMethod(AB3_Schaf.class, "satt", 7);
            schafe.get(0).dreheLinks();
            for(int i=0; i<3;i++) {
                AB3_Schaf s = schafe.get(i);
                s.verbraucheEnergie(s.getHunger()-(70-i*12));
                m.invoke(s);
                if(s.getHunger()<80) {
                    melde("FEHLER: Das Schaf hat nicht genug gefressen.");
                    return;
                }
            }
            melde("Teste sattFressen(): OK.",1500);

            melde("Teste faules Schaf ... ", 1000);
            m = getMethod(AB3_Schaf.class, "faul", 9);
            for(int i=0; i<3;i++) {
                AB3_Schaf s = schafe.get(i);
                s.verbraucheEnergie(s.getHunger()-(70-i*12),s.getDurst()-80);
                m.invoke(s);
                if(s.getHunger()<80) {
                    melde("FEHLER: Das Schaf hat nicht genug gefressen.");
                    return;
                }
            }
            melde("Teste faules Schaf: OK.",1500);

            melde("Teste zurck zum Stall ... ", 1000);
            m = getMethod(AB3_Schaf.class, "Stall", 10);
            Random r = new Random();
            for(int i=3; i<5;i++) {
                AB3_Schaf s = schafe.get(i);
                int x = r.nextInt(6)+6;
                while(s.getRotation()!=0) s.dreheLinks();
                while(s.getX()!=x) s.einsVor();
                s.dreheRechts();
                s.dreheRechts();
            }

            for(int i=3; i<6;i++) {
                AB3_Schaf s = schafe.get(i);
                s.verbraucheEnergie(s.getHunger()-70,s.getDurst()-80);
                m.invoke(s);
                if(s.getX()!=6) {
                    melde("FEHLER: Die X-Position des Schafs stimmt noch nicht.");
                    return;
                }
                if(s.getY()!=3) {
                    melde("FEHLER: Die Y-Position des Schafs stimmt noch nicht.");
                    return;
                }
            }
            melde("Teste zurck zum Stall: OK.",1500);
        } catch(Throwable e) {
            if(m != null) {
                melde("FEHLER: Die Methode "+m.getName()+" wurde nicht richtig deklariert.");
            }
            return;
        }

        melde("Alle Tests wurden bestanden.\nLevel 4 freigeschaltet.");
        secret.maxLevel(4);
    }

    public void leveltest_03()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,1200,  0,  0,  0,  0},
                {200,200,200,200,200,200,  0,  0,  0,  0,  0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0}
            };

        weltLeeren();
        Random r = new Random();
        backg[10][1+r.nextInt(6)] = 100;
        backg[10][7+r.nextInt(6)] = 300;
        generateWorld(backg);
        zeichneUeberschrift("Leveltest 3: Karges Land");
        AB3_Schaf ab3 = new AB3_Schaf();
        addObject(ab3,   1, 10);
        ab3.verbraucheEnergie(90,20);

        ab3.leveltest3();
        Greenfoot.delay(2);
        if (ab3.getWorld()==null && ab3.getX()==6 && ab3.getY()==3) {
            melde("Super! Das Schaf ist wohlbehalten im Stall angekommen.");
        } else {
            melde("Du hast die Aufgabe noch nicht erfllt");
        }
        Greenfoot.setWorld(new Bauernhofprojekt());

    }

    private void ab4_verzweiger()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,1200, 0,  0,  0},
                {203,200,200,200,200,200,  0,  0,  0,  0,  0,  0,  0,  0},
                {205,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0},
                {205,  0,203,200,200,  0,  0,  0,  0,  0,  0,  0,  0,205},
                {205,104,205,  0,  0,  0,104,  0,308,307,307,307,306,205},
                {205,104,205,  0,  0,  0,203,200,309,310,310,310,304,205},
                {205,  0,205,104,200,200,202,1303,302,305,303,310,304,205},
                {205,  0,205,104,  0,1303,104,1303,104,104,302,305,301,205},
                {209,200,202,  0,205,1302,104,104,1303,1303,1303,1303,104,205},
                {205,104,  0,  0,205,1300,104,1303,104,1303,1303,104,1303,205},
                {201,200,200,200,208,200,200,200,200,200,200,200,200,202}}; 

        weltLeeren();
        generateWorld(backg);
        zeichneUeberschrift("AB4 - Verzweigungen nutzen");

        AB4_Schaf f;
        f = new AB4_Schaf();
        addObject(f, 1, 8);
        f.dreheLinks();

        f = new AB4_Schaf();
        addObject(f, 1, 10);
        f.dreheLinks();

        f = new AB4_Schaf();
        addObject(f, 9, 8);
        f.dreheLinks();

        f = new AB4_Schaf();
        addObject(f, 5, 10);
    }

    public void checkup_04()
    {
        ab4_verzweiger();
        List<AB4_Schaf> schafe = this.getObjects(AB4_Schaf.class);
        melde("Teste dreheRichtigrum()...",1000);
        for(int i=0; i<4;i++) {
            AB4_Schaf s = schafe.get(i);
            savePosition(s);
            if(s.istRechtsFrei()) {
                s.dreheRichtigrum();
                if(!checkPosition(s,90)) return;
            } else {
                s.dreheRichtigrum();
                if(!checkPosition(s,-90)) return;
            }
        }
        melde("Teste dreheRichtigrum(): OK.",1500);

        melde("Teste geradeausOderDrehen()...",1000);
        AB4_Schaf s = schafe.get(0);
        savePosition(s);
        s.geradeausOderDrehen();
        if(!checkPosition(s,90,0,0)) return;
        s.geradeausOderDrehen();
        if(!checkPosition(s,90,0,-1)) return;

        s = schafe.get(1);
        savePosition(s);
        s.geradeausOderDrehen();
        if(!checkPosition(s,0,1,0)) return;
        s.geradeausOderDrehen();
        if(!checkPosition(s,0,2,0)) return;
        s.geradeausOderDrehen();
        if(!checkPosition(s,-90,2,0)) return;

        s = schafe.get(2);
        savePosition(s);
        s.geradeausOderDrehen();
        if(!checkPosition(s,0,-1,0)) return;
        s.geradeausOderDrehen();
        if(!checkPosition(s,0,-2,0)) return;
        s.geradeausOderDrehen();
        if(!checkPosition(s,0,-3,0)) return;

        s = schafe.get(3);
        savePosition(s);
        s.geradeausOderDrehen();
        if(!checkPosition(s,0,0,-1)) return;
        s.geradeausOderDrehen();
        if(!checkPosition(s,0,0,-2)) return;
        s.geradeausOderDrehen();
        if(!checkPosition(s,90,0,-2)) return;
        melde("Teste geradeausOderDrehen(): OK.",1500);

        Method m=null;
        try{
            melde("Teste zurck in den Stall ...",1000);
            while(schafe.get(2).istVorneFrei()) schafe.get(2).einsVor();
            schafe.get(2).dreheRechts();
            m = getMethod(AB4_Schaf.class, "Stall", 5);
            if(m == null) return;
            for(int i=0; i<3;i++) {
                s = schafe.get(i);
                m.invoke(s);

                if(s.getX()!=7) {
                    melde("FEHLER: Die X-Position des Schafs stimmt noch nicht.");
                    return;
                }
                if(s.getY()!=3) {
                    melde("FEHLER: Die Y-Position des Schafs stimmt noch nicht.");
                    return;
                }

            }

            Gras g = getObjectsAt(6, 5, Gras.class).get(0);
            if(g.getWert() == 15) { melde("FEHLER: Das Gras wird unterwegs nicht gefressen."); return; }
            g = getObjectsAt(3, 7, Gras.class).get(0);
            if(g.getWert() == 15) { melde("FEHLER: Das Gras wird unterwegs nicht gefressen."); return; }
            g = getObjectsAt(3, 8, Gras.class).get(0);
            if(g.getWert() == 15) { melde("FEHLER: Das Gras am Startpunkt wird nicht gefressen.\nVermutlich lufst das Schaf zuerst bevor es testet, ob Gras da ist."); return;}
            melde("Teste zurck in den Stall: OK.",1500);

            melde("Teste lecker Bluemchen() ...",1000);
            m = getMethod(AB4_Schaf.class, "Bluemchen", 7);
            s = schafe.get(3);
            s.dreheRechts();
            s.einsVor();
            s.dreheLinks();
            m.invoke(s);
            s.dreheLinks();
            s.trinke();
            s.trinke();
            s.trinke();
            s.dreheRechts();
            s.dreheRechts();
            s.einsVor();
            s.dreheRechts();
            m.invoke(s);
            if(s.getX()!=5) {
                melde("FEHLER: Die X-Position des Schafs stimmt noch nicht.");
                return;
            }
            if(s.getY()!=10) {
                melde("FEHLER: Die Y-Position des Schafs stimmt noch nicht.");
                return;
            }
            List<Blume> bl = getObjects(Blume.class);
            for(Blume b: bl) {
                if(b.getWert()>0 && b.getY() == 9 && b.getX()==5) {
                    melde("FEHLER: Es wurden die Blume am Startpunkt nicht gefressen.\nMan braucht eine if-Anweisung vor der Schleife.");
                    return;
                }

                if(b.getWert()>0 && b.getY() == 10 && b.getX()==5) {
                    melde("FEHLER: Es wurden die Blume am Endpunkt nicht gefressen.\nMan braucht eine if-Anweisung nach der Schleife.");
                    return;
                }

                if(b.getWert()>0 && b.getY() >= 9) {
                    melde("FEHLER: Es wurden nicht alle leckeren Blumen gefressen.");
                    return;
                }
            }
            melde("Teste lecker Bluemchen(): OK.",1500);

        } catch(Throwable e) {
            if(m != null) {
                melde("FEHLER: Die Methode "+m.getName()+" wurde nicht richtig deklariert.");
            }
            return;
        }

        melde("Alle Tests wurden bestanden.\nLevel 5 freigeschaltet.");
        secret.maxLevel(5);
    }    

    public void leveltest_04()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,1200, 0,  0,  0},
                {203,200,200,200,200,200,  0,  0,  0,  0,  0,  0,  0,  0},
                {205,300,1303, 0,  0,104,  0,  0,  0,  0,  0,  0,  0,  0},
                {205,  0,  0,200,200,  0,  0,  0,200,200,207,200,200,204},
                {205,104,  0,  0,1303,308,307,306,  0,  0,205,  0,300,205},
                {209,200,200,200,  0,309,310,304,  0,1303, 0,  0,104,205},
                {205,  0,  0,1303, 0,302,305,301,  0,  0,205,  0,  0,205},
                {209,200,  0,200,204,  0,  0,  0,203,  0,208,200,200,206},
                {205,  0,104,  0,205,  0,  0,104,  0, 0,1303,  0,104,205},
                {205,300,  0,  0,  0,1303, 0, 0,205,1303, 0,104,300,205},
                {201,200,200,200,208,200,200,200,208,200,200,200,200,202}};

        weltLeeren();
        generateWorld(backg);
        zeichneUeberschrift("Leveltest 4: Durchs Labyrinth");
        AB4_Schaf f;
        f = new AB4_Schaf();
        Random r = new Random();
        int x=0;
        int y=4;
        while(backg[y][x]>=200 && backg[y][x]< 400) {
            x = r.nextInt(13);
            y = 5+r.nextInt(5);
        }
        addObject(f, x, y);
        f.setRotation(r.nextInt(4)*90);
        f.verbraucheEnergie(60,70);

        f.leveltest4();
        Greenfoot.delay(2);
        if (f.getWorld()==null && f.getX()==7 && f.getY()==3) {
            secret.writeLevel(5);
            melde("Super! Das Schaf hat den Weg durch das Labyrinth gefunden.");
        } else {
            melde("Du hast die Aufgabe noch nicht erfllt");
        }
        Greenfoot.setWorld(new Bauernhofprojekt());
    }

    private void ab5_funktionen()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,1200, 0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {203,200,200,200,200,200,  0,  0,  0,  0,  0,  0,  0, 0},
                {205,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 300,205},
                {205,  0,700,700,700,700,  0,  0,  0,  0,  0,  0,  0,205},
                {205,  0,700,700,700,700,  0,  0,700,700,700,700,  0,205},
                {205,  0,700,700,700,700,  0,300,700,700,700,700,  0,205},
                {205,  0,700,700,700,700,  0,  0,700,700,700,700,  0,205},
                {205,  0,700,700,700,700,  0,  0,700,700,700,700,  0,205},
                {205,300,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,205},
                {201,200,200,200,200,200,200,200,200,200,200,200,200,202}};

        weltLeeren();
        generateWorld(backg);

        zeichneUeberschrift("AB5 - Methoden mit Rckgabewert");

        AB5_Bauer ab5;
        ab5 = new AB5_Bauer();
        addObject(ab5,  11, 4);
        ab5.addAnzahlVonGegenstand("Tomate", 5);
        ab5.setRotation(90);
        ab5.verbraucheEnergie(70,30);

        ab5 = new AB5_Bauer();
        addObject(ab5,  8, 7);
        ab5.addAnzahlVonGegenstand("Tomate", 5);
        ab5.setRotation(90);
        ab5.verbraucheEnergie(40,40);

        ab5 = new AB5_Bauer();
        addObject(ab5,  5, 5);
        ab5.addAnzahlVonGegenstand("Giesskanne",1);
        ab5.getGegenstandAusInventar("Giesskanne").setzeBild(3);
        ab5.setRotation(180);
        ab5.verbraucheEnergie(20,80);

        ab5 = new AB5_Bauer();
        addObject(ab5,  4, 9);
        ab5.addAnzahlVonGegenstand("Giesskanne",1);
        ab5.getGegenstandAusInventar("Giesskanne").setzeBild(0);
        ab5.verbraucheEnergie(5,15);

        for(int x=2; x < 6; x++) {
            for(int y=5; y<10;y++) {
                Tomate t = new Tomate();
                addObject(t, x,y);
                t.setzeBild(y-5);
                if((x+y)%2==0) t.giesse();
            }

        }

    }

    public void checkup_05()
    {
        ab5_funktionen();
        List<AB5_Bauer> bauern = this.getObjects(AB5_Bauer.class);
        AB5_Bauer b = null;

        melde("Teste istAufReiferTomate()...",200);

        for(int i=0; i<4;i++) {
            b= bauern.get(i);
            for(int j=0; j<2; j++){
                boolean erg1 = (b.istAuf("Tomate") && b.pruefe("Tomate") == 100);
                boolean erg2 = b.istAufReiferTomate();
                if(erg1 != erg2){
                    melde("FEHLER Bauer "+(i+1)+": Das Ergebnis ist "+ erg2+ " statt "+  erg1);
                    return;
                }   
                else {
                    melde("Bauer "+(i+1)+": Ist reif: "+erg2,100);
                }

                b.einsVor();

            }
        } 
        melde("Teste istAufReiferTomate(): OK.",1500);

        Method m=null;
        try{
            melde("Teste durstige Tomate ...",1000);
            m = getMethod(AB5_Bauer.class, "durstig", 4);
            if(m == null) return;

            for(int i=0; i<4;i++) {
                b= bauern.get(i);
                b.dreheRechts();
                b.dreheRechts();
                for(int j=0; j<2; j++){
                    boolean erg1 = (b.istAuf("Tomate") && b.pruefe("Tomate") < 0);
                    Boolean erg2 = (Boolean) m.invoke(b);
                    if(erg1 != erg2){
                        melde("FEHLER Bauer "+(i+1)+": Das Ergebnis ist "+ erg2+ " statt "+  erg1);
                        return;
                    }   
                    else {
                        melde("Bauer "+(i+1)+": Tomate ist durstig: "+erg2,100);
                    }
                    b.einsVor();

                }
            } 
            melde("Teste durstige Tomate: OK.",1500);

            melde("Teste Energie schwach ...",1000);

            m = getMethod(AB5_Bauer.class, "Energie", 5);
            if(m == null) return;

            for(int i=0; i<4;i++) {
                b= bauern.get(i);
                boolean erg1 = (b.getDurst() < 40 || b.getHunger() <40);
                Boolean erg2 = (Boolean) m.invoke(b);
                if(erg1 != erg2){
                    melde("FEHLER Bauer "+(i+1)+": Das Ergebnis ist "+ erg2+ " statt "+  erg1);
                    return;
                }   
                else {
                    melde("Bauer "+(i+1)+": Ist Energie schwach: "+erg2,300);
                }
            }                   
            melde("Teste Energie schwach: OK.",1500);

            melde("Teste, ob Bauer gieen kann ...",1000);

            m = getMethod(AB5_Bauer.class, "kannGiessen", 7);
            if(m == null) return;

            for(int i=0; i<4;i++) {
                b= bauern.get(i);
                boolean erg1 = (b.hatGegenstand("Giesskanne") && b.pruefe("Giesskanne") > 0);
                Boolean erg2 = (Boolean) m.invoke(b);
                if(erg1 != erg2){
                    melde("FEHLER Bauer "+(i+1)+": Das Ergebnis ist "+ erg2+ " statt "+  erg1);
                    return;
                }   
                else {
                    melde("Bauer "+(i+1)+": kann gieen: "+erg2,300);
                }
            }      
            melde("Teste, ob Bauer gieen kann: OK.",1500);

            melde("Teste istLinks ...",1000);
            m = getMethod(AB5_Bauer.class, "istLinks", 8);
            if(m == null) return;

            b= bauern.get(0);
            savePosition(b);
            if((Boolean) m.invoke(b,"Wasser")) {
                melde("FEHLER: Links von Bauer 1 ist kein Wasser");
                return;
            }          
            if(!checkPosition(b)) return;

            b= bauern.get(1);
            savePosition(b);
            if(!((Boolean) m.invoke(b,"Wasser"))) {
                melde("FEHLER: Links von Bauer 2 ist Wasser.");
                return;
            }          
            if(!checkPosition(b)) return;
            if((Boolean) m.invoke(b,"Acker")) {
                melde("FEHLER: Links von Bauer 1 ist kein Acker");
                return;
            }          
            if(!checkPosition(b)) return;

            b= bauern.get(3);
            b.dreheLinks();
            b.dreheLinks();
            savePosition(b);
            if(!((Boolean) m.invoke(b,"Tomate"))) {
                melde("FEHLER: Links von Bauer 3 ist eine Tomate.");
                return;
            }          
            if(!((Boolean) m.invoke(b,"Acker"))) {
                melde("FEHLER: Links von Bauer 3 ist eine Tomate.");
                return;
            }          
            if(!checkPosition(b)) return;
            melde("Teste istLinks: OK.",1500);

            melde("Teste laufeBisY ...",1000);
            m = getMethod(AB5_Bauer.class, "laufeBisY", 9);
            if(m == null) return;

            b= bauern.get(0);
            b.dreheLinks();
            b.dreheLinks();
            savePosition(b);
            m.invoke(b, 6);
            if(!checkPosition(b,0,2)) return;
            m.invoke(b,10);
            if(!checkPosition(b,0,6)) return;
            b=bauern.get(1);
            savePosition(b);
            m.invoke(b,4);
            if(!checkPosition(b,0,-3)) return;

            melde("Teste laufeBisY: OK.",1500);

        } catch(Throwable e) {
            if(m != null) {
                melde("Die Methode "+m.getName()+" wurde nicht richtig deklariert.");
            }
            return;
        }

        melde("Alle Tests wurden bestanden.\nLevel 6 freigeschaltet.");
        secret.maxLevel(6);
    }    

    public void leveltest_05()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,1200, 0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {203,200,200,200,200,200,  0,  0,  0,  0,  0,  0,  0, 0},
                {205,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,1100,205},
                {205,  0,700,700,700,700,  0,300,  0,  0,  0,  0, 300,205},
                {205,  0,700,700,700,700,  0,  0,700,700,700,700,  0,205},
                {205,  0,700,700,700,700,  0,  0,700,700,700,700,  0,205},
                {205,  0,700,700,700,700,  0,  0,700,700,700,700,  0,205},
                {205,  0,700,700,700,700,  0,  0,700,700,700,700,  0,205},
                {205,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,205},
                {201,200,200,200,200,200,200,200,200,200,200,200,200,202}};

        weltLeeren();
        generateWorld(backg);
        zeichneUeberschrift("Leveltest 5: Gemsebauernprfung");

        AB5_Bauer ab5;
        ab5 = new AB5_Bauer();
        addObject(ab5,  11, 4);
        ab5.addAnzahlVonGegenstand("Tomate", 4);
        ab5.setRotation(90);

        ab5.leveltest5();
        Greenfoot.delay(2);
        if(ab5.getAnzahl("Tomate")>=20) {
            secret.writeLevel(6);
            melde("Du hast die Gemsebauernprfung bestanden und\n"+ab5.getAnzahl("Tomate")+" Tomaten in "+time+" Schritten geerntet.");
        } else {
            melde("Du hast die Aufgabe noch nicht erfllt");
        }

        Greenfoot.setWorld(new Bauernhofprojekt());
    }

    private void ab6_attribute()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,1200, 0, 0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {203,200,200,200,200,200,  0,  0,  0,  0,  0,  0,0,200},
                {205,400,  0,400,  0,  0,600,  0,  0,  0,  0,  0,  0, 0},
                {205,  0,400,  0,205,600,  0,  0,  0,  0,  0,400,300, 0},
                {205,400,  0,600,201,204,  0,  0,  0,  0,  0,400,308,307},
                {205,600,  0,  0,600,205,  0,  0,  0,  0,  0,400,309,310},
                {205,104,600,104,  0,205,  0,  0,  0,  0,  0,400,309,310},
                {205,600,  0,  0,  0,205,  0,  0,  0,  0,  0,400,309,310},
                {205,  0,104,203,200,208,200,200,200,200,200,200,302,305},
                {201,200,200,202,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0}};

        weltLeeren();
        generateWorld(backg);

        zeichneUeberschrift("AB6 - Eigene Attribute");
        AB6_Huhn huhn;
        huhn = new AB6_Huhn();
        addObject(huhn, 1, 4);
        huhn.setRotation(90);
        huhn.verbraucheEnergie(30, 10);
        huhn = new AB6_Huhn();
        addObject(huhn, 2, 5);
        huhn.setRotation(90);
        huhn.verbraucheEnergie(50, 20);
        huhn = new AB6_Huhn();
        addObject(huhn, 3, 4);
        huhn.verbraucheEnergie(70, 30);
        huhn.setRotation(90);
        huhn = new AB6_Huhn();
        addObject(huhn, 1, 6);
        huhn.verbraucheEnergie(70, 30);
        huhn.setRotation(90);
        AB6_Bauer bauer;
        bauer = new AB6_Bauer();
        addObject(bauer,10,4);
        bauer.addAnzahlVonGegenstand("Koerner", 100);
        bauer.setRotation(90);
        Random r = new Random();
        for(int x = 7; x < 11; x++) {
            for(int y = 5; y <10; y ++) {
                if(r.nextInt(10) < 3) {
                    addObject(Gegenstand.erzeuge("Koerner"),x,y);
                }
            }
        }
        for(int y = 5; y <10; y ++) {
            if(r.nextInt(10) < 3) {
                addObject(Gegenstand.erzeuge("Ei"),11,y);
            }
        }

    }

    public void checkup_06()
    {
        ab6_attribute();
        List<AB6_Huhn> huehner = this.getObjects(AB6_Huhn.class);
        AB6_Huhn h = null;

        melde("Checke unntige Attribute...", 1000);
        Field[] fields = AB6_Huhn.class.getDeclaredFields();

        for(Field f: fields) {
            if(!f.getName().toLowerCase().equals("koernergefressen") &&
            !f.getName().toLowerCase().contains("ei") &&
            !f.getName().toLowerCase().contains("entfernung")) {

                melde("FEHLER: Ein Attribut '"+f.getName()+"' darf es nicht geben.\nberprfe die Benennung.");
                return;

            }
        }
        if(fields.length  > 3) {
            melde("FEHLER: Du hast zu viele Attribute deklariert.\nberprfe die Aufgaben.");
            return;
        }
        melde("Checke unntige Attribute: OK.", 1500);

        Field f = null;
        Method m = null;
        try{
            melde("Teste sucheKoerner()...",1000);

            for(Field f1: fields) {
                if(f1.getName().toLowerCase().contains("entfernung")) {
                    f = f1;
                    break;
                }
            }
            if(f == null) {
                melde("FEHLER: Das Attribut entfernungZumNest fehlt.\n Bitte deklariere es.");
                return;
            }
            if(!f.getType().getName().equals("int")){
                melde("FEHLER: Das Attribut "+f.getName()+" sollte vom Typ int statt vom Typ "+f.getType().getName()+" sein.");
                return;
            }

            int[] entf = {5,3,2,1,6,5,5,4};
            for(int j=3; j>=0; j--) {
                h = huehner.get(j);
                h.sucheKoerner();
                f.setAccessible(true);
                if(((Integer) f.get(h))!= entf[j]) {
                    melde("FEHLER: Das Huhn "+(j+1)+" zhlt  die Entfernung falsch");
                    return;
                }
                if(h.istAuf("Koerner")) {
                    melde("FEHLER: Die Krner wurden nicht gefressen.");
                }
            }
            melde("Teste sucheKoerner(): OK.",1500);

            melde("Teste gibEntfernungZumNest()...",1000);
            Method m2 = getMethod(AB6_Huhn.class, "gibEntfernungZumNest", 7);
            if(m2==null) return;
            for(int j=3; j>=0; j--) {
                h = huehner.get(j);
                if((Integer) m2.invoke(h) != entf[j]) {
                    melde("FEHLER: die Methode gibEntfernungZumNest gibt einen falschen Wert zurck.");
                    return;
                }
                else {
                    melde("Huhn "+(j+1)+" hat Entfernung "+entf[j],300);
                }
            }
            melde("Teste gibEntfernungZumNest(): OK.",1500);

            melde("Teste sucheKoerner() ohne Korn...",1000);
            for(int j=3; j>=0; j--) {
                h = huehner.get(j);
                h.sucheKoerner();
                f.setAccessible(true);
                if(((Integer) f.get(h))!= entf[j+4]) {
                    melde("FEHLER: Das Huhn "+(j+1)+" zhlt  die Entfernung falsch");
                    return;
                }

                if(h.istVorneFrei()) {
                    melde("FEHLER: Vorne ist noch frei.");
                }
            }
            melde("Teste sucheKoerner() ohne Korn: OK.",1500);

            melde("Teste laufeZurueckZumNest()...",1000);
            for(int j=3; j>=0; j--) {
                h = huehner.get(j);
                savePosition(h);
                h.laufeZurueckZumNest();

                if(!checkPosition(h,180,0,-entf[j+4])) {
                    melde("FEHLER: das Huhn "+(j+1)+" luft nicht zu seinem Nest zurck.");
                    return;
                }
                if((Integer) m2.invoke(h) != 0) {
                    melde("FEHLER: die Entfernung zum Nest msste jetzt 0 sein.");
                    return;
                }
            }
            h = huehner.get(2);
            h.dreheRechts();
            savePosition(h);
            h.sucheKoerner();
            h.laufeZurueckZumNest();
            if(!checkPosition(h, 180)) {
                melde("FEHLER: beim 2. Mal luft das Huhn nicht zu seinem Nest zurck.\nZhlst du auch Schritte nach rechts?");
                return;
            }
            melde("Teste laufeZurueckZumNest(): OK.",1500);

            melde("Teste picken()...",1000);
            for(int y = 6; y< 10;y++) {

                Gegenstand k = Gegenstand.erzeuge("Koerner");
                addObject(k, 2, y);
            }

            h = huehner.get(1);
            h.dreheRechts();
            h.dreheRechts();
            savePosition(h);
            h.picken();
            if(!checkPosition(h, 180)) {
                melde("FEHLER: Nach der Methode picken, msste das Huhn wieder auf dem Nest sein.");
                return;
            }

            for(int y = 6; y <10; y++) {
                List<Gegenstand> k = getObjectsAt(2,y, Gegenstand.class);
                if(k != null && k.size() > 0) {
                    melde("FEHLER: Nach picken() mssten alle Krner der Reihe weg sein.");
                    return;
                }
            }

            melde("Teste picken(): OK.",1500);

            melde("Teste Eier legen...",1000);
            for(Field f1: fields) {
                if(f1.getName().toLowerCase().equals("eigelegt")) {
                    f = f1;
                    break;
                }
            }
            if(f == null) {
                melde("FEHLER: Das Attribut eiGelegt fehlt.\n Bitte deklariere es.");
                return;
            }
            if(!f.getType().getName().equals("boolean")){
                melde("FEHLER: Das Attribut "+f.getName()+" sollte vom Typ boolean statt vom Typ "+f.getType().getName()+" sein.");
                return;
            }
            f.setAccessible(true);
            huehner.get(0).dreheRechts();
            for(Huhn h1 : huehner) {
                if(((Boolean) f.get(h1))) {
                    melde("FEHLER: Am Anfang muss das Attribut eiGelegt auf false stehen.");
                    return;
                }
                m = getMethod(AB6_Huhn.class, "gackere", 10);
                if(m==null) return;
                m2 = getMethod(AB6_Huhn.class, "hatEiGelegt", 10);
                if(m2 == null) return;

                boolean ei = false;
                for(int i=0; i<2; i++) {
                    m.invoke(h1);
                    if(h1.istAuf("Ei")) { ei = true;}
                    if(((Boolean) f.get(h1)) != ei) {
                        melde("FEHLER: das Attribut eiGelegt hat den Wert "+((Boolean) f.get(h1))+" statt "+ei);
                        return;
                    }
                    if(((Boolean) m2.invoke(h1)) != ei) {
                        melde("FEHLER: das Attribut eiGelegt stimmt, aber hatEiGelegt gibt den falschen Wert zurck");
                        return;
                    }
                }
                h1.einsVor();
                m.invoke(h1);
                if(((Boolean) f.get(h1)) != ei) {
                    melde("FEHLER: das Attribut eiGelegt hat den Wert "+((Boolean) f.get(h1))+" statt "+ei);
                    return;
                }
                if(((Boolean) m2.invoke(h1)) != ei) {
                    melde("FEHLER: das Attribut eiGelegt stimmt, aber hatEiGelegt gibt den falschen Wert zurck");
                    return;
                }
            }

        } catch(Throwable e) {
            if(m != null) {
                melde("FEHLER: Die Methode "+m.getName()+" wurde nicht richtig deklariert.");
            }
            if(f != null) {
                melde("FEHLER: Der Zugriff auf "+f.getName()+" schlgt fehlt.");

            }
            return;
        }

        melde("Alle Tests wurden bestanden.\nLevel 7 freigeschaltet.");
        secret.maxLevel(7);
    }    

    public void leveltest_06()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,1200, 0, 0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {203,200,200,200,200,200,  0,  0,  0,  0,  0,  0,0,200},
                {205,400,  0,400,  0,  0,600,  0,  0,  0,  0,  0,  0, 0},
                {205,  0,400,  0,205,600,  0,  0,  0,  0,  0,400,300, 0},
                {205,  0,  0,600,201,204,  0,  0,  0,  0,  0,400,308,307},
                {205,600,  0,  0,600,205,  0,  0,  0,  0,  0,400,309,310},
                {205,104,600,104,  0,205,  0,  0,  0,  0,  0,400,309,310},
                {205,600,  0,  0,  0,205,  0,  0,  0,  0,  0,400,309,310},
                {205,  0,104,203,200,208,200,200,200,200,200,200,302,305},
                {201,200,200,202,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0}};

        weltLeeren();
        generateWorld(backg);

        zeichneUeberschrift("AB6 - Hhnerbauerprfung");

        Random r = new Random();

        for(int y=5; y<10; y++) {
            AB6_Huhn huhn1;
            huhn1 = new AB6_Huhn();
            addObject(huhn1, 11, y);
            huhn1.setRotation(180);
            huhn1.verbraucheEnergie(30+r.nextInt(50), 10+r.nextInt(50));
            huhn1.istHauptperson = false;
            addAktion(()->huhn1.automatischeBewegung());
        }

        AB6_Bauer bauer;
        bauer = new AB6_Bauer();
        addObject(bauer,10,4);
        bauer.addAnzahlVonGegenstand("Koerner", 100);
        bauer.setRotation(90);
        for(int x = 7; x < 11; x++) {
            for(int y = 5; y <10; y ++) {
                if(r.nextInt(10) < 8) {
                    addObject(Gegenstand.erzeuge("Koerner"),x,y);
                }
            }
        }
        int ei = -1;
        while(bauer.getAnzahl("Ei") < 30 && bauer.getAnzahl("Ei")>ei) {
            ei = bauer.getAnzahl("Ei");
            bauer.leveltest6();
        }

        if(bauer.getAnzahl("Ei") >= 30 && bauer.getAnzahl("Ei") == bauer.getAnzahlEier()) {
            secret.writeLevel(7);
            melde("Hhnerbauernprfung erfolgreich bestanden.\nDu hast "+ bauer.getAnzahl("Ei")+" Eier in "+time+" Schritten eingesammelt.");
        } else {
            melde("Du hast die Aufgabe noch nicht erfllt");
        }
        Greenfoot.setWorld(new Bauernhofprojekt());
    }

    private void ab7_lokale_variablen_zaehlschleifen()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,1200, 0, 0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {203,200,200,207,200,200,  0,  0,  0,  0,  0,  0,  0,204},
                {205,700,700,205,700,700,700,205,700,700,700,700,700,205},
                {205,700,700,205,700,700,700,205,700,700,700,700,700,205},
                {205,700,700,205,700,700,700,205,700,700,700,700,700,205},
                {205,308,307,306,700,700,700,205,700,700,700,700,700,205},
                {205,309,310,304,700,700,700,205,700,700,700,700,700,205},
                {205,302,305,301,200,200,200,208,200,200,200,200,200,206},
                {205,700,700,700,700,700,700,700,700,700,700,700,700,205},
                {201,200,200,200,200,200,200,200,200,200,200,200,200,202}};

        weltLeeren();
        generateWorld(backg);

        zeichneUeberschrift("AB7 - lokale Variablen, Zhlschleifen");

        Random r = new Random();
        int anzBlumen = 0;
        for(int x = 1; x < 13; x++) {
            int z = r.nextInt(10);
            if(z < 7) {
                if(z < 5) {
                    addObject(Gegenstand.erzeuge("Blume"), x, 10);
                    anzBlumen++;
                } else {
                    addObject(Gegenstand.erzeuge("Gras"), x,10);
                }
            }
        }
        while(anzBlumen < 3) {
            int x = r.nextInt(12)+1;
            if(getObjectsAt(x,10, Blume.class).size()==0) {
                for(Actor a : getObjectsAt(x,10, Gras.class)) {
                    removeObject(a);
                }
                addObject(Gegenstand.erzeuge("Blume"), x, 10);
                anzBlumen++;

            }
        }

        for(int x = 4; x < 7; x++) {
            for(int y = 4; y < 9; y++) {
                int z = r.nextInt(10);
                if(z < 4) {
                    Gegenstand g = Gegenstand.erzeuge("Tomate");
                    g.setzeBild(4);
                    addObject(g, x, y);
                }
            }
        }

        AB7_Bauer bauer;

        bauer = new AB7_Bauer();
        addObject(bauer,  1, 4);
        bauer.setRotation(90);
        bauer.addAnzahlVonGegenstand("Koerner", 100);

        bauer = new AB7_Bauer();
        addObject(bauer,  4, 4);
        bauer.setRotation(90);
        bauer.addAnzahlVonGegenstand("Koerner", 100);

        bauer = new AB7_Bauer();
        addObject(bauer,  8, 4);
        bauer.setRotation(90);
        bauer.addAnzahlVonGegenstand("Koerner", 100);

        bauer = new AB7_Bauer();
        addObject(bauer,  1, 10);
        bauer.addAnzahlVonGegenstand("Koerner", 100);

    }

    public void checkup_07()
    {
        ab7_lokale_variablen_zaehlschleifen();
        List<AB7_Bauer> bauern = this.getObjects(AB7_Bauer.class);
        AB7_Bauer b = null;

        melde("Checke unntige Attribute...", 1000);
        Field[] fields = AB7_Bauer.class.getDeclaredFields();

        for(Field f: fields) {
            if(!f.getName().toLowerCase().equals("anzahl1")) {

                melde("FEHLER: Ein Attribut '"+f.getName()+"' darf es nicht geben.");
                return;

            }
        }
        if(fields.length  > 1) {
            melde("FEHLER: Du hast zu viele Attribute deklariert.\nDu sollst hier lokale Variabeln verwenden.");
            return;
        }
        melde("Checke unntige Attribute: OK.", 1500);

        Field f = null;
        Method m = null;
        try{
            melde("Teste bestimmeLaenge()...",1000);

            int[] entf = {3,5,5,12};
            for(int j=0; j<4; j++) {
                b = bauern.get(j);
                int l = b.bestimmeLaenge();
                if(l == entf[j]-1) {
                    melde("FEHLER: Der Bauer "+(j+1)+" bestimmt die Entfernung falsch.\nDenk daran, dass sowohl Start- als auch Zielfeld mitgezhlt werden mssen.");
                    return;
                }
                if(l != entf[j]) {
                    melde("FEHLER: Der Bauer "+(j+1)+" bestimmt die Entfernung falsch.");
                    return;
                }
                melde("Der Bauer "+(j+1)+" bestimmt die Entfernung "+entf[j],300);
                b.dreheRechts();
                b.dreheRechts();
            }
            melde("Teste bestimmeLaenge(): OK.",1500);

            melde("Teste bestimmeLaenge2()...",1000);

            for(int j=0; j<3; j++) {
                b = bauern.get(j);
                int l = b.bestimmeLaenge2();
                if(l == entf[j]-1) {
                    melde("FEHLER: Der Bauer "+(j+1)+" bestimmt die Entfernung falsch.\nDenk daran, dass sowohl Start- als auch Zielfeld mitgezhlt werden mssen.");
                    return;
                }
                if(l != entf[j]) {
                    melde("FEHLER: Der Bauer "+(j+1)+" bestimmt die Entfernung falsch.");
                    return;
                }
                melde("Der Bauer "+(j+1)+" bestimmt die Entfernung "+entf[j],300);
                b.dreheRechts();
                b.dreheRechts();
            }
            melde("Teste bestimmeLaenge2(): OK.",1500);

            melde("Teste bestimmeFlaeche()...",1000);
            int[] flaechen = {6,15,25,12};

            m = getMethod(AB7_Bauer.class, "Flaeche", 7);
            if(m==null) return;
            for(int j=3; j>=0; j--) {
                b = bauern.get(j);
                int fl = ((Integer) m.invoke(b));
                if(fl != flaechen[j]) {
                    melde("FEHLER: die Methode bestimmeFlaeche gibt "+fl+" statt "+flaechen[j]+" zurck.");
                    return;
                }
                else {
                    melde("Bauer "+(j+1)+" hat eine Feldgre "+flaechen[j],300);
                }
            }
            melde("Teste bestimmeFlaeche(): OK.",1500);

            melde("Teste zhle Unkraut...",1000);

            m = getMethod(AB7_Bauer.class, "Unkraut", 8);
            if(m==null) return;

            b = bauern.get(3);
            b.setRotation(0); 
            removeObjects(getObjectsAt(1,10,Gegenstand.class));
            removeObjects(getObjectsAt(12,10,Gegenstand.class));
            addObject(Gegenstand.erzeuge("Acker"),1,10);
            addObject(Gegenstand.erzeuge("Acker"),12,10);
            addObject(Gegenstand.erzeuge("Gras"),1,10);
            addObject(Gegenstand.erzeuge("Gras"),12,10);

            int a = (Integer) m.invoke(b);
            int c = getObjects(Gras.class).size();
            if(a == c-1) {
                melde("FEHLER: Die Anzahl der Grasfelder wird nicht richtig bestimmt.\nDu ermittelst "+a+" statt "+c+".\nHast du an das Start- und an das Endfeld gedacht?");
                return;
            }
            if(a != c) {
                melde("FEHLER: Die Anzahl der Grasfelder wird nicht richtig bestimmt.\nDu ermittelst "+a+" statt "+c+".");
                return;
            }
            melde("Teste zhle Unkraut: OK.",1500);

            melde("Teste zhle Schritte bis zur 3. Blume...",1000);

            m = getMethod(AB7_Bauer.class, "Blume", 9);
            if(m==null) return;

            b = bauern.get(3);
            b.dreheLinks();
            b.dreheLinks();

            a = (Integer) m.invoke(b);
            c = 12;
            int x = 3;
            int y = 0;
            while(x>0) {
                if(getObjectsAt(c,10, Blume.class).size()>0) {
                    if(y == 0) y = c;
                    x--;
                }
                c--;
            }

            if(a != 11-c) {
                melde("FEHLER: Die Anzahl der Schritte wird nicht richtig bestimmt.\nDu ermittelst "+a+" statt "+(11-c)+".");
                return;
            }
            b.dreheLinks();
            b.dreheLinks();
            a = (Integer) m.invoke(b);

            if(a != y-c-1) {
                melde("FEHLER: Die Anzahl der Schritte wird nicht richtig bestimmt.\nDu ermittelst "+a+" statt "+(y-c-1)+".\nHast du daran gedacht, dass er auf einer Blume starten kann?");
                return;
            }

            melde("Teste zhle Schritte bis zur 3. Blume: OK.",1500);

            melde("Teste pflanzeKorn2...",1000);
            b = bauern.get(2);
            b.dreheLinks();
            y = 0;
            for(x = 5; x>0; x--) {
                b.pflanzeKorn2(x);
                y += x;
                if(y != getObjects(Korn.class).size()) {
                    melde("FEHLER: Du pflanzt nicht die richtige Anzahl von Korn");
                    return;
                }
                b.dreheLinks();
                b.einsVor();
            }

            melde("Teste pflanzeKorn2: OK.",1500);

            melde("Teste Drehwurm...",1000);
            m = getMethod(AB7_Bauer.class, "drehwurm", 13);
            if(m==null) return;
            for(x = 0; x< 4; x++) {
                b = bauern.get(x);
                y = b.getHunger();
                m.invoke(b, x+1);
                if(y-(x+1)*8< b.getHunger()) {
                    melde("FEHLER: Der Bauer dreht nicht oft genug.");
                    return;
                }
                if(y-(x+1)*8 > b.getHunger()) {
                    melde("FEHLER: Der Bauer dreht zu oft.");
                    return;
                }
            }

            melde("Teste Drehwurm: OK.",1500);

            melde("Teste pflanze Kornfeld...",1000);
            ab7_lokale_variablen_zaehlschleifen();
            bauern = this.getObjects(AB7_Bauer.class);
            b = bauern.get(0);

            b.kornfeld(3,2);
            if(getObjects(Korn.class).size() != 6) {
                melde("FEHLER: Der Bauer pflanzt nicht 6 Kornfelder");
                return;
            }

            b = bauern.get(2);

            b.kornfeld(4,5);
            if(getObjects(Korn.class).size() != 26) {
                melde("FEHLER: Der Bauer pflanzt nicht 20 Kornfelder");
                return;
            }

            melde("Teste pflanze Kornfeld: OK.",1500);
        } catch(Throwable e) {
            if(m != null) {
                System.out.println(e);
                melde("FEHLER: Die Methode "+m.getName()+" wurde nicht richtig deklariert.");
            }
            if(f != null) {
                melde("FEHLER: Der Zugriff auf "+f.getName()+" schlgt fehlt.");

            }
            return;
        }

        melde("Alle Tests wurden bestanden.\nLevel 8 freigeschaltet.");
        secret.maxLevel(8);
    }       

    public void leveltest_07()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,1200, 0, 0,  0,  0,  0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {203,200,200,200,200,200,  0,  0,  0,  0,  0,  0,  0,204},
                {205,700,700,700,700,700,700,700,700,700,700,700,700,205},
                {205,700,700,700,700,700,700,700,700,700,700,700,700,205},
                {205,700,700,700,700,700,700,700,700,700,700,700,700,205},
                {205,700,700,700,700,700,700,700,700,700,700,700,700,205},
                {205,700,700,700,700,700,700,700,700,700,700,700,700,205},
                {205,700,700,700,700,700,700,700,700,700,700,700,700,205},
                {205,700,700,700,700,700,700,700,700,700,700,700,700,205},
                {201,200,200,200,200,200,200,200,200,200,200,200,200,202}};

        Random r = new Random();
        int breite = 0;
        int hoehe =  0;
        while((hoehe-1)*breite < 20 || (hoehe-1)*breite > 30){
            breite = r.nextInt(4)*2+5;
            hoehe =  r.nextInt(6)+2;
            if(hoehe == 7) hoehe = 8;
            if(breite==11) breite = 9;
        }

        for(int x = 13-breite; x <13; x++) backg[3+hoehe][x] = 200;
        for(int y = 4; y <11; y++) backg[y][12-breite] = 205;
        backg[3+hoehe][12-breite]= 209;
        backg[11][12-breite] = 208;
        backg[3+hoehe][13] = 206;
        if(backg[3][12-breite] != 0) backg[3][12-breite]= 207;
        backg[4][13-breite-1] = 1400;

        weltLeeren();
        generateWorld(backg);

        zeichneUeberschrift("Leveltest 7: Kornbauernprfung");

        AB7_Bauer bauer;
        bauer = new AB7_Bauer();
        addObject(bauer,  13-breite, 4);
        bauer.setRotation(90);
        bauer.addAnzahlVonGegenstand("Koerner", 70);
        Greenfoot.delay(20);
        bauer.leveltest7();

        if(bauer.getAnzahl("Koerner") >= 300) {
            secret.writeLevel(8);
            melde("Kornbauernprfung erfolgreich bestanden.\nDu hast "+ bauer.getAnzahl("Koerner")+" Koerner in "+time+" Schritten geerntet.");
        } else {
            melde("Du hast die Aufgabe noch nicht erfllt");
        }
        Greenfoot.setWorld(new Bauernhofprojekt());
    }

    private void ab8_arrays()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,1200,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,  0,  0,  0},
                {203,200,200,300,200,200,  0,  0,  0,  0,  0,  0,  0,204},
                {205,400,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 400,205},
                {205,400,  0,  0,  0,  0,  0,300,  0,  0,  0,  0, 400,205},
                {205,400,300,  0,  0,  0,  0,  0,  0,  0,  0,300, 400,205},
                {205,400,400,400,400,400,400,  0,  0,300,  0,  0, 400,205},
                {209,200,200,200,200,300,400,  0,  0,  0,  0,  0, 400,205},
                {205,1303,1303,1303,1303,205,400,  0,  0,  0,300,  0,400,205},
                {205,1303,1303,1303,1303,205,400,300,  0,  0,  0,  0,400,205},
                {205,1303,1303,1303,1303,205,400,400,400,400,400,400,400,205},
                {201,200,200,200,200,208,200,200,200,200,200,200,200,202}};

        weltLeeren();
        generateWorld(backg);

        zeichneUeberschrift("Level 8 - Arrays");

        Random r = new Random();

        for(int i=0; i<5; i++) {
            if(r.nextInt(10) < 5) {
                int x=0;
                int y=0;
                boolean ok = false;
                while(!ok) {
                    x = r.nextInt(12)+1;
                    y = r.nextInt(7)+4;
                    ok = true;
                    if(getObjectsAt(x,y, Gegenstand.class).size() > 0) ok = false;
                }

                Huhn huhn1;
                huhn1 = new Huhn();
                addObject(huhn1, x, y);

                huhn1.setRotation(r.nextInt(4)*90);
                huhn1.istHauptperson = false;
                addAktion(()->huhn1.automatischeBewegung2());

            }
        }

        Gehilfe gehilfe = new Gehilfe();
        addAktion(()->gehilfe.automatischeBewegung());
        addObject(gehilfe,3,5);

        AB8_Bauer bauer;
        bauer = new AB8_Bauer();
        addObject(bauer,10,3);
        bauer.addAnzahlVonGegenstand("Koerner", 100);
        bauer.setRotation(90);

        bauer = new AB8_Bauer();
        addObject(bauer,1,8);
        bauer.setRotation(90);

        for(int x = 1; x < 11; x++) {
            for(int y = 3; y <10; y ++) {
                if(getObjectsAt(x,y,Gegenstand.class).size()==0 && r.nextInt(10) < 8) {
                    addObject(Gegenstand.erzeuge("Koerner"),x,y);
                }
            }
        }

        addAktion(()->blumenwachstum());
    }

    int[] anzBlumen2= new int[3];

    public void checkup_08()
    {
        ab8_arrays();
        aktionen.clear();
        addAktion(()->blumenwachstum());

        List<AB8_Bauer> bauern = this.getObjects(AB8_Bauer.class);
        AB8_Bauer b = null;

        melde("Checke unntige Attribute...", 1000);
        Field[] fields = AB8_Bauer.class.getDeclaredFields();

        for(Field f: fields) {
            if(!f.getName().toLowerCase().equals("anzahlblumen") &&
            !f.getName().toLowerCase().equals("anzahleier") &&
            !f.getName().toLowerCase().equals("nestnr")  &&
            !f.getName().toLowerCase().contains("name")) {

                melde("FEHLER: Ein Attribut '"+f.getName()+"' darf es nicht geben.");
                return;

            }
        }
        if(fields.length  > 4) {
            melde("FEHLER: Du hast zu viele Attribute deklariert.");
            return;
        }
        melde("Checke unntige Attribute: OK.", 1500);

        Field f = null;
        Method m = null;
        try{
            b = bauern.get(1);
            melde("Teste Initialisierung des Array anzahlBlumen...",1000);
            for(Field f2: fields) {
                if(f2.getName().toLowerCase().equals("anzahlblumen")) {
                    f = f2;
                    break;
                }
            }
            if(f==null) {
                melde("FEHLER: Ich finde das Attribut anzahlBlumen nicht.");
                return;
            }
            f.setAccessible(true);
            int[] anzBlumen = (int[]) f.get(b);
            if(anzBlumen.length != 3) {
                melde("FEHLER: Das Array anzahlBlumen hat nicht die Lnge 3.");
                return;
            }
            for(int i=0; i< 3; i++) {
                if(anzBlumen[i] != 0) {
                    melde("FEHLER: Der "+i+". Eintrag hat nicht den Wert 0.");
                    return;
                }
            }
            melde("Teste Initialisierung des Array anzahlBlumen: OK.",1500);
            f= null;

            melde("Teste pflueckeBlumen()...", 1000);
            anzBlumen2[0] = 0;
            anzBlumen2[1] = 0;
            anzBlumen2[2] = 0;

            List<Blume> blumen = getObjects(Blume.class);
            for(Blume bl : blumen) {
                if(bl.getWert() == -40) anzBlumen2[0]++;
                if(bl.getWert() == 30) anzBlumen2[1]++;
                if(bl.getWert() == 40) anzBlumen2[2]++;
            }
            b.pflueckeBlumen();
            blumen = getObjects(Blume.class);
            for(Blume bl : blumen) {
                if(bl.getWert() == -40) anzBlumen2[0]--;
                if(bl.getWert() == 30) anzBlumen2[1]--;
                if(bl.getWert() == 40) anzBlumen2[2]--;
            }
            b.pflueckeBlumen();
            for(int i = 0; i<3; i++) {
                if(anzBlumen[i] != anzBlumen2[i]) {
                    melde("FEHLER: Die Anzahl der Blumen (Position "+i+") stimmt nicht");
                    return;
                }
            }
            melde("Teste pflueckeBlumen(): OK.", 1500);
            melde("Teste gibGesamtzahlBlumen()...", 1000);
            m = getMethod(AB8_Bauer.class, "gibGesamtzahlBlumen", 3);
            int anz1 = (Integer) m.invoke(b);
            int anz2 = anzBlumen2[0]+anzBlumen2[1]+anzBlumen2[2];
            if(anz1 != anz2) {
                melde("FEHLER: Die Gesamtzahl der Blumen stimmt nicht.\n Es sollte "+anz2+" statt "+anz1+" herauskommen."); 
                return;
            }
            melde("Teste gibGesamtzahlBlumen(): OK.", 1500);
            melde("Teste gibAnzahlBlumenart(int art)...", 1000);
            m = getMethod(AB8_Bauer.class, "gibAnzahlBlumenart", 4);
            for(int i = 0; i< 3; i++) {
                int anz3 = (Integer) m.invoke(b,i);
                int anz4 = anzBlumen2[i];
                if(anz3 != anz4) {
                    melde("FEHLER: Die Zahl der Blumen bei art="+i+" stimmt nicht.\n Es sollte "+anz4+" statt "+anz3+" herauskommen."); 
                    return;
                } 
            }
            melde("Teste gibAnzahlBlumenart(int art): OK.", 1500);

            melde("Teste gibProzentsatz(int art)...", 1000);
            m = getMethod(AB8_Bauer.class, "gibProzentsatz", 5);
            for(int i = 0; i< 3; i++) {
                double p1 = (Double) m.invoke(b,i);
                double p2 = (double) anzBlumen2[i]/anz2;
                if(p1 != p2) {
                    melde("FEHLER: Der Prozentsatz der Blumen bei art="+i+" stimmt nicht.\n Es sollte "+p2+" statt "+p1+" herauskommen."); 
                    return;
                } 
            }
            melde("Teste gibProzentsatz(int art): OK.", 1500);

            melde("Teste gibNrHaeufigsteBlume()...", 1000);
            m = getMethod(AB8_Bauer.class, "gibNrHaeufigsteBlume", 6);
            int haeufigste = 0;
            if(anzBlumen2[1] > anzBlumen2[0]) haeufigste = 1;
            if(anzBlumen2[2] > anzBlumen2[haeufigste]) haeufigste = 2;
            int haeufigste2 = (Integer) m.invoke(b);
            if(haeufigste != haeufigste2) {
                melde("FEHLER: Die hufigste Blume hat die Nummer "+haeufigste+" und nicht "+haeufigste2); 
                return;
            } 
            melde("Teste gibNrHaeufigsteBlume(): OK.", 1500);
            melde("Teste gibAnzahlHaeufigsteBlume()...", 1000);
            m = getMethod(AB8_Bauer.class, "gibAnzahlHaeufigsteBlume", 5);
            int anzhaeufigste2 = (Integer) m.invoke(b);
            if(anzBlumen2[haeufigste] != anzhaeufigste2) {
                melde("FEHLER: Die hufigste Blume gibt es "+anzBlumen2[haeufigste]+" mal und nicht "+anzhaeufigste2); 
                return;
            } 
            melde("Teste gibAnzahlHaeufigsteBlume(): OK.", 1500);

            //-------------------- Eier ------------------------
            b = bauern.get(0);
            melde("Teste Initialisierung des Array anzahlEier...",1000);
            for(Field f2: fields) {
                if(f2.getName().toLowerCase().equals("anzahleier")) {
                    f = f2;
                    break;
                }
            }
            if(f==null) {
                melde("FEHLER: Ich finde das Attribut anzahlEier nicht.");
                return;
            }
            f.setAccessible(true);
            int[] anzEier = (int[]) f.get(b);
            if(anzEier.length != 26) {
                melde("FEHLER: Das Array anzahlEier hat nicht die Lnge 26.");
                return;
            }
            for(int i=0; i< 26; i++) {
                if(anzEier[i] != 0) {
                    melde("FEHLER: Der "+i+". Eintrag hat nicht den Wert 0.");
                    return;
                }
            }
            melde("Teste Initialisierung des Array anzahlEier: OK.",1500);
            f= null;
            melde("Teste Attribut nestNr...",1000);
            for(Field f2: fields) {
                if(f2.getName().toLowerCase().equals("nestnr")) {
                    f = f2;
                    break;
                }
            }
            if(f==null) {
                melde("FEHLER: Ich finde das Attribut nestNr nicht.");
                return;
            }
            f.setAccessible(true);
            melde("Teste Attribut nestNr: OK.",1500);
            int anzGegenstaende = getObjects(Gegenstand.class).size();
            addObject(Gegenstand.erzeuge("Ei"), 12, 3);
            addObject(Gegenstand.erzeuge("Ei"), 12, 4);
            addObject(Gegenstand.erzeuge("Ei"), 12, 6);
            addObject(Gegenstand.erzeuge("Ei"), 12, 7);

            melde("Teste laufeSchritte()...", 1000);
            b.dreheLinks();
            savePosition(b);
            b.laufeSchritte(2);
            if (!checkPosition(b, 2, 0)) return;
            int nr = (Integer) f.get(b);
            if (nr != 1) {
                melde("Fehler: NestNr wird nicht richtig gezhlt.\n Es kommt "+nr+" statt 1 heraus.");
                return;
            }
            if (anzEier[0]!= 1) {
                melde("Fehler: anzahlEier[0] msste jetzt 1 sein. Es ist aber "+anzEier[0]);
                return;
            }                
            b.dreheRechts();
            savePosition(b);
            b.laufeSchritte(0);
            if (!checkPosition(b, 0, 0)) return;
            nr = (Integer) f.get(b);
            if (nr != 1) {
                melde("Fehler: NestNr wird nicht richtig gezhlt.\n Es kommt "+nr+" statt 1 heraus.");
                return;
            }
            if (anzEier[0]!= 1) {
                melde("Fehler: anzahlEier[0] msste jetzt 1 sein. Es ist aber "+anzEier[0]);
                return;
            }                
            b.laufeSchritte(5);
            if (!checkPosition(b, 0, 5)) return;
            nr = (Integer) f.get(b);
            if (nr != 6) {
                melde("Fehler: NestNr wird nicht richtig gezhlt.\n Es kommt "+nr+" statt 6 heraus.");
                return;
            }
            if (anzEier[1]!= 1) {
                melde("Fehler: anzahlEier[1] msste jetzt 1 sein. Es ist aber "+anzEier[1]);
                return;
            }              
            if (anzEier[2]!= 0) {
                melde("Fehler: anzahlEier[2] msste jetzt 0 sein. Es ist aber "+anzEier[2]);
                return;
            }                

            if (anzEier[3]!= 1) {
                melde("Fehler: anzahlEier[3] msste jetzt 1 sein. Es ist aber "+anzEier[3]);
                return;
            }                
            if (anzEier[4]!= 1) {
                melde("Fehler: anzahlEier[4] msste jetzt 1 sein. Es ist aber "+anzEier[4]);
                return;
            }                
            if( getObjects(Gegenstand.class).size() != anzGegenstaende) {
                melde("Fehler: die Eier werden nicht eingesammelt. Es liegen noch "+(getObjects(Gegenstand.class).size()-anzGegenstaende)+" da.");
                return;
            }
            melde("Teste laufeSchritte(): OK.", 1500);

            melde("Teste gibGesamtzahlEier()...", 1000);
            m = getMethod(AB8_Bauer.class, "gibGesamtzahlEier", 12);
            anz1 = (Integer) m.invoke(b);
            anz2 = 4;
            if(anz1 != anz2) {
                melde("FEHLER: Die Gesamtzahl der Eier stimmt nicht.\n Es sollte "+anz2+" statt "+anz1+" herauskommen."); 
                return;
            }
            melde("Teste gibGesamtzahlEier(): OK.", 1500);
            melde("Teste gibAnzahlEierInNest()...", 1000);
            m = getMethod(AB8_Bauer.class, "gibAnzahlEierInNest", 11);
            int anz3 = (Integer) m.invoke(b,0);
            int anz4 = 1;
            if(anz3 != anz4) {
                melde("FEHLER: Die Zahl der Eier in Nest=0 stimmt nicht.\n Es sollte "+anz4+" statt "+anz3+" herauskommen."); 
                return;
            } 
            anz3 = (Integer) m.invoke(b,1);
            anz4 = 1;
            if(anz3 != anz4) {
                melde("FEHLER: Die Zahl der Eier in Nest=1 stimmt nicht.\n Es sollte "+anz4+" statt "+anz3+" herauskommen."); 
                return;
            } 
            anz3 = (Integer) m.invoke(b,2);
            anz4 = 0;
            if(anz3 != anz4) {
                melde("FEHLER: Die Zahl der Eier in Nest=2 stimmt nicht.\n Es sollte "+anz4+" statt "+anz3+" herauskommen."); 
                return;
            } 
            melde("Teste gibAnzahlEierInNest(): OK.", 1500);

            melde("Teste gibDurchschnitt()...", 1000);
            m = getMethod(AB8_Bauer.class, "gibDurchschnitt", 13);
            double p1 = (Double) m.invoke(b);
            double p2 = 4.0/26;
            if(p1 != p2) {
                melde("FEHLER: Der durchschnittliche Eierertrag stimmt nicht.\n Es sollte "+p2+" statt "+p1+" herauskommen."); 
                return;
            } 
            melde("Teste gibDurchschnitt(): OK.", 1500);

            melde("Teste nutzlose Nester...", 1000);
            m = getMethod(AB8_Bauer.class, "leer", 14);
            int leer = (Integer) m.invoke(b);
            if(leer != 22) {
                melde("FEHLER: Es mssten 22 leere Nester sein. Du ermittelst +"+leer); 
                return;
            } 
            melde("Teste nutzlose Nester: OK.", 1500);

            melde("Teste Lieblingsnest...", 1000);
            m = getMethod(AB8_Bauer.class, "gibNrBestesNest", 15);
            haeufigste2 = (Integer) m.invoke(b);
            if(haeufigste2 > 4 || haeufigste2 == 3) {
                melde("FEHLER: Das Lieblingsnest wird nicht richtig ermittelt. "+haeufigste2+" stimmt nicht.");
                return;
            }
            m = getMethod(AB8_Bauer.class, "gibAnzahlEierBestesNest", 15);
            anzhaeufigste2 = (Integer) m.invoke(b);
            if(1 != anzhaeufigste2) {
                melde("FEHLER: Es gab nur max. 1 Ei pro Nest, nicht "+anzhaeufigste2); 
                return;
            } 
            melde("Teste Lieblingsnest: OK.", 1500);

        } catch(Throwable e) {
            if(m != null) {
                System.out.println(e);
                melde("FEHLER: Die Methode "+m.getName()+" wurde nicht richtig deklariert.");
            }
            if(f != null) {
                melde("FEHLER: Der Zugriff auf "+f.getName()+" schlgt fehlt.");

            }
            melde("Unbekannter Fehler: "+e);
            return;
        }

        melde("Alle Tests wurden bestanden.\nDu hast alle Level geschafft.");
        secret.maxLevel(9);
    }       

    public void blumenwachstum() {
        List<Gegenstand> liste;
        for (int x = 1; x < 5; x++) {
            for(int y = 8; y <11; y++) {

                liste = getObjectsAt(x,y,Gegenstand.class);
                if(liste.size() == 0) {
                    Gegenstand gras = Gegenstand.erzeuge("Gras");
                    gras.setzeBild(0);
                    addObject(gras,x,y);
                } else {
                    Gegenstand g = liste.get(0);
                    if(g.getName().toUpperCase().equals("GRAS") && g.getWert() == 15) {
                        removeObject(g);
                        Gegenstand blume = Gegenstand.erzeuge("Blume");
                        addObject(blume,x,y);
                        if(blume.getWert() == -40) anzBlumen2[0]++;
                        if(blume.getWert() == 30) anzBlumen2[1]++;
                        if(blume.getWert() == 40) anzBlumen2[2]++;

                    }
                }
            }
        }
    }

    public void koernerverteilung() {
        Random r = new Random();
        for(int x = 1; x < 13; x++) {
            for(int y = 3; y <11; y++) {
                List<Gegenstand> l = getObjectsAt(x,y,Gegenstand.class);
                if(l.size() == 0 && r.nextInt(50)<1) {
                    addObject(Gegenstand.erzeuge("Koerner"), x,y);
                }
            }
        }
    }

    public void leveltest_08()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,1200,  0,  0,  0,  0, 0},
                {  0,  0,  0,  0,  0,  0,  0,  0,  0, 0, 0,  0,  0,  0},
                {203,200,200,200,207,200,  0,  0,  0,  0,  0,  0,  0,204},
                {205,  0,400,  0,104,  0,400,  0,104,600,  0,600, 400,205},
                {205,  0,300,  0,205,  0,600,600,205,  0,600,300,   0,205},
                {205,600,  0,600,205,600,  0,300,205,600,  0,  0, 400,205},
                {205,400,600,  0,205,  0,  0,  0,209,104,200,200,200,206},
                {209,104,200,200,208,207,200,200,206,600,  0,600,  0,205},
                {205,600,  0,  0,  0,205,600,  0,209,200,300,204,400,205},
                {205,400,300,600,  0,104,600,300,205,  0,600,103,600,205},
                {205,  0,  0,  0,  0,205,400,600,104,600,400,205,300,205},
                {201,200,200,200,200,208,200,200,200,200,200,208,200,202}};

        weltLeeren();
        generateWorld(backg);

        zeichneUeberschrift("Leveltest 8: Agrarkonom");

        Random r = new Random();

        Huhn huhn1;
        huhn1 = new Huhn(190);
        addObject(huhn1, 2, 6);

        huhn1.setRotation(r.nextInt(4)*90);
        huhn1.istHauptperson = false;
        addAktion(()->huhn1.automatischeBewegung2());

        Huhn huhn2 = new Huhn(240);
        addObject(huhn2, 5, 4);

        huhn2.setRotation(r.nextInt(4)*90);
        huhn2.istHauptperson = false;
        addAktion(()->huhn2.automatischeBewegung2());

        Huhn   huhn3 = new Huhn(130);
        addObject(huhn3, 9, 5);

        huhn3.setRotation(r.nextInt(4)*90);
        huhn3.istHauptperson = false;
        addAktion(()->huhn3.automatischeBewegung2());

        Huhn huhn4;
        huhn4 = new Huhn(300);
        addObject(huhn4, 2, 10);

        huhn4.setRotation(r.nextInt(4)*90);
        huhn4.istHauptperson = false;
        addAktion(()->huhn4.automatischeBewegung2());

        Huhn huhn5 = new Huhn(200);
        addObject(huhn5, 6, 8);

        huhn5.setRotation(r.nextInt(4)*90);
        huhn5.istHauptperson = false;
        addAktion(()->huhn5.automatischeBewegung2());

        Huhn   huhn6 = new Huhn(350);
        addObject(huhn6, 9, 10);

        huhn6.setRotation(r.nextInt(4)*90);
        huhn6.istHauptperson = false;
        addAktion(()->huhn6.automatischeBewegung2());

        Huhn   huhn7 = new Huhn(80);
        addObject(huhn7, 12, 7);

        huhn7.setRotation(r.nextInt(4)*90);
        huhn7.istHauptperson = false;
        addAktion(()->huhn7.automatischeBewegung2());
        AB8_Bauer ab8_bauer;
        ab8_bauer = new AB8_Bauer();
        addObject(ab8_bauer,   10, 3);
        ab8_bauer.setRotation(90);

        addAktion(()->koernerverteilung());

        String s = ab8_bauer.leveltest8();
        melde(s);

        Pattern p = Pattern.compile("[+-]?[0-9]+");
        Matcher m = p.matcher(s);

        int z = -1;
        int anz = 0;
        boolean ok = true;
        try{
            while ( m.find() ) {
                int y = Integer.parseInt(m.group());
                if(y >= z) {
                    z = y;
                    anz++;
                } else {
                    ok =false;
                    break;
                }
            }
        } catch(Exception e) {ok = false;}

        Greenfoot.delay(2);

        if (ok && anz==7) {
            secret.writeLevel(9);
            melde("Du hast die Rangfolge deiner Hhner gefunden.");
        } else {
            melde("Du hast die Aufgabe noch nicht erfllt.\nDie Hhner sind nicht richtig sortiert.");
        }
        Greenfoot.setWorld(new Bauernhofprojekt());
    }

    private void ab9_finalBild()
    {
        int[][] backg = {{  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0, 0},
                {104,104,104,104,104,104,104,104,104,104,104,104,104,104},
                {104,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,104},
                {104,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,104},
                {104,1301,1301,1301,  0,1302,  0,  0,1300,  0,  0,1301,  0,104},
                {104,  0,1301,  0,1302,  0,1302,  0,1300,  0,  0,1301,  0,104},
                {104,  0,1301,  0,1302,  0,1302,  0,1300,  0,  0,1301,  0,104},
                {104,  0,1301,  0,1302,  0,1302,  0,1300,  0,  0,1301,  0,104},
                {104,  0,1301,  0,  0,1302,  0,  0,1300,1300,  0,1301,1301,104},
                {104,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,104},
                {104,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,  0,104},
                {104,104,104,104,104,104,104,104,104,104,104,104,104,104}};

        weltLeeren();

        generateWorld(backg);
        addObject(new Schaf(), 2, 2);
        addObject(new Huhn(), 10,3);
        addObject(new Huhn(), 3, 10);
        addObject(new Bauer(), 7,9);
        addObject(new Schaf(), 9,10);
        zeichneUeberschrift("Bravo, jetzt kannst du programmieren!");

    }

    private void melde(String text) {
        melde(text,3000);   
    }

    private void melde(String text, int ms)
    {

        Message m = new Message(text);
        addObject(m, 7,6);
        long z = System.currentTimeMillis();
        while(System.currentTimeMillis()-z < ms){
            Greenfoot.delay(1);
        }
        removeObject(m);

    }

    // public <A> List<A> getObjectsAt(int x, int y, Class<A> c) {
    // List<A> l = super.getObjectsAt(x, y, c);
    // l.removeIf(a->!(((Actor)a).getX()==x && ((Actor)a).getY()==y));
    // return l;
    // }
}
