import greenfoot.*;
import java.util.*;
import javax.swing.JOptionPane;
import java.io.File;  

/** Die Klasse Gegenstand fasst alle in der Welt liegenden Actors zusammen.
 * Teilweise sind sie bewegbar und/oder betretbar. Der Wert
 * des Gegenstandes gibt in der Regel an, wie viel Energie er
 * bringt.
 * 
 * @author T. Schaller 
 * @version 04.10.2021
 */
public class Gegenstand extends MyActor
{
    protected int wert;
    // private int typ;
    // private int[] schaden = {0, 0, 0, 0, 20, 0, 50, 40, 0, 0, 0, 0, 0, 0, 0};
    // private int[] lebensdauer = {1, 1, 1, 1, 0, 1000, 0, 0, 5, 1000, 1000,1, 1000, 1000,1000}; 
   
    private static int[] defwert = {-1 , -1, 1000000, -1, 1, 60, -1, -1, -1, 0, 3, -1, 0, -1};
    private static boolean[] defmoveable = {false, false,false, false, true, true, false, false, false, false, true, false, true, true };
    private static boolean[] defbetretbar = {true, false, false, true, true, true, true, true, false, false, true, false, true, true};
    private static String[] typen = {"Gras","Zaun","Wasser","Nest","Ei","Koerner","Acker","Korn","Stall","Tomate", "Giesskanne","Bauernhof", "Blume", "Sense"};
    /** untersucht, ob name eine korrekte Bezeichnug fr einen Gegenstand ist
     * @return erg true, wenn name korrekt ist
     */
    public static boolean istGegenstand(String name) {
        boolean erg = false;
        for(int i = 0; i < typen.length; i++) {
            if (name.toUpperCase().equals(typen[i].toUpperCase())) {
                erg = true;
            }
        }
        return erg;
    }

    /** untersucht, ob name eine korrekte Bezeichnug fr einen Gegenstand ist
     * @return erg true, wenn name korrekt ist
     */
    public static int getTypGegenstand(String name) {
        for(int i = 0; i < typen.length; i++) {
            if (name.toUpperCase().equals(typen[i].toUpperCase())) {
                return i;
            }
        }
        return -1;
    }

    public static Gegenstand erzeuge(String name) {
        return erzeuge(getTypGegenstand(name));
    }

    public static Gegenstand erzeuge(int nr) {
        if(nr>=0 && nr < typen.length) {
            if(typen[nr].equals("Gras")) return new Gras();
            if(typen[nr].equals("Korn")) return new Korn();
            if(typen[nr].equals("Tomate")) return new Tomate();
            if(typen[nr].equals("Bauernhof")) return new Bauernhof();
            if(typen[nr].equals("Blume")) return new Blume();
            if(typen[nr].equals("Giesskanne")) return new Giesskanne();
            return new Gegenstand(typen[nr], defmoveable[nr], defbetretbar[nr], defwert[nr]);
        }
        return null;
    }

    
    /** erzeugt einen Standardgegenstand 
     */
    public Gegenstand(String name, boolean moveable, boolean betretbar, int wert) {
        super(name, moveable, betretbar);
        this.wert = wert;
    }


    /** liefert den Wert des Gegenstands
     * @return Wert je nach Gegenstand unterschiedlich
     */
    public int getWert() {
        return wert;
    }

 

    /**
     * setLocation(x,y) fuer Gegenstand wird ueberschrieben,
     * damit der Gegenstand nur auf Felder gesetzt werden kann,
     * wo dies zulssig ist.
     */
    public void setLocation(int x, int y) {
        List<MyActor> g = getWorld().getObjectsAt(x,y, MyActor.class);

        for(MyActor a : g) {
            if(!(a.istBetretbar(x, y))) return;
        }
        super.setLocation(x, y);
    }


    /**
     * setRotation(x) wird verhindert, da die Gegenstnde sich 
     * nicht drehen knnen.
     */
    public void setRotation(int x) {
        // nichts machen!!
    }


}