import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot und MouseInfo)
import java.util.*;
/**
 * Die Klasse Huhn stellt eine programmierbare Figur dar, die
 * als Besonderheit auf Nestern Eier legen kann. Je besser genhrt
 * desto hher ist die Wahrscheinlichkeit fr ein Ei.
 * 
 * @author T. Schaller 
 * @version 04.10.2021
 */
public class Huhn extends Figur
{
    private int quote;  // Quote fr erfolgreiches Ei (0 = immer, 
                        // je grer desto seltener
    
    public Huhn() {
        super("Huhn");
        quote = 110;
    }
    
    public Huhn(int quote) {
        super("Huhn");
        this.quote = quote;
    }

    /**
     * Diese Methode lsst das Huhn ein Korn fressen, 
     * wenn es auf dem gleichen Feld steht.
     */
    public void fresse() {
        if(getWorld() == null) return;
        Gegenstand g = gibGegenstandUnterFigur("Koerner");
        if(g!= null) {
            setzeBild(bildnummer+4);
            Greenfoot.delay(3);
            hunger += g.getWert();
            if(hunger>100) hunger =100;
            getWorld().removeObject(g);
            setzeBild(bildnummer-4);
        }
        else {
            warne("Da ist nichts zum Fressen!",this);
        }

    }

    /** Diese Methode lsst das Huhn etwas trinken,
     * wenn es vor einem Feld mit Wasser steht.
     */
    public void trinke() {
        if(getWorld() == null) return;

        Gegenstand g = gibGegenstandVorFigur("Wasser");

        if(g!= null) {
            setzeBild(bildnummer+4);
            Greenfoot.delay(3);
            durst += 25;
            if(durst>100) durst =100;
            setzeBild(bildnummer-4);
        } else {
            warne("Da ist nichts zum Trinken!",this);
        }
    }

    /** Das Huhn versucht ein Ei zu legen, falls es auf
     * einem leeren Nest steht. Je besser genhrt es ist, desto 
     * wahrscheinlicher ist der Erfolg.
     */
    public void versucheEiZuLegen() {
        if(getWorld() == null) return;
        Gegenstand nest = gibGegenstandUnterFigur("Nest");
        Gegenstand ei = gibGegenstandUnterFigur("Ei");
        Random r = new Random();
        if(nest!=null && ei == null) {
            if(verbraucheEnergie(25,15)) {
                setzeBild(bildnummer+8);
                Greenfoot.delay(5);
                setzeBild(bildnummer-8);
                if(r.nextInt(quote) < getHunger()){
                    getWorld().addObject(Gegenstand.erzeuge("Ei"), getX(), getY() );
                } 
            } else {
                warne("Ich habe nicht genug Kraft, um ein Ei zu legen",this);
            }

        } else {
            warne("Ich kann nur in ein leeres Nest Eier legen", this);
        }
    }

    /** 
     * Hilfsmethode fr Level 6: Bitte nicht aufrufen
     */
    public void automatischeBewegung() {
        Random r = new Random();
        if(getRotation()==180) {
            if(istAuf("Nest")) {
                if(!istAuf("Ei") && getHunger()>60) versucheEiZuLegen();
                einsVor();
            } else {
                if(getX() == 10 && getWorld().getObjectsAt(11, getY(), Gegenstand.class).size()>1) {
                    if(getDurst() <20) dreheRechts(); 
                    else warte();
                } else {
                    if(getHunger() > 60) {
                        dreheRechts();
                    } else {
                        if(istAuf("Koerner") && r.nextInt(10)<4) {
                            fresse();
                        }
                        if(istVorneFrei()) {
                            einsVor();
                        } else {
                            dreheRechts();
                        }
                    }
                }
            }
        } else {
            if(getRotation()==0) {
                if(istVorne("Wasser")) {
                    if(getDurst() <100) trinke();
                    else dreheRechts();
                }
                else {
                    if(istVorneFrei()) einsVor();
                }
            }
            else dreheRechts();
        }
    }

    /**
     * Hilfsmethode: testet, ob rechts name ist
     * @param name Bezeichnung des Gegenstands.
     */
    private boolean istRechts(String name)
    {
        if(getWorld() == null) { error(); return false;}

        int lRichtung = getRotation();

        int neuX = getX();
        int neuY = getY();
        if (lRichtung == 0)  neuY++;  
        if (lRichtung == 90)  neuX--;
        if (lRichtung == 180)  neuY--;
        if (lRichtung == 270)  neuX++;
        List<Gegenstand> liste = new ArrayList<Gegenstand>();
        liste.addAll(getWorld().getObjectsAt(neuX, neuY,Gegenstand.class));

        for(Gegenstand a : liste) {
            if(a.getName().toUpperCase().equals(name.toUpperCase())) return true;
        } 
        return false;
    }

    /**
     * Hilfsmethode: testet, ob links name ist
     * @param name Bezeichnung des Gegenstands.
     */
    private boolean istLinks(String name)
    {
        if(getWorld() == null) { error(); return false;}

        int lRichtung = getRotation();

        int neuX = getX();
        int neuY = getY();
        if (lRichtung == 0)  neuY--;  
        if (lRichtung == 90)  neuX++;
        if (lRichtung == 180)  neuY++;
        if (lRichtung == 270)  neuX--;
        List<Gegenstand> liste = new ArrayList<Gegenstand>();
        liste.addAll(getWorld().getObjectsAt(neuX, neuY,Gegenstand.class));

        for(Gegenstand a : liste) {
            if(a.getName().toUpperCase().equals(name.toUpperCase())) return true;
        } 
        return false;
    }

    /**
     * Hilfsmethode fr Level 8: bitte nicht aufrufen
     */
    public void automatischeBewegung2() {

        Random r = new Random();
        if(istAuf("Nest")) {
            if(!istAuf("Ei") && getHunger()>60) {
                versucheEiZuLegen();
            }
        }
        boolean laufe = true;
        if(istVorne("Wasser") && getDurst() < 80) { 
            trinke();
            if(getDurst() < 80) laufe = false;
        }

        if(istAuf("Koerner") && getHunger() < 80) fresse();

        if(laufe) {
            if(istRechts("Wasser") && getDurst() < 70) {
                dreheRechts();
            } else {
                if(istLinks("Wasser") && getDurst() < 70) {
                    dreheLinks();
                } else {
                    if(istVorne("Koerner") && getHunger() < 60) {
                        einsVor();
                    } else {
                        if(istRechts("Koerner") && getHunger() < 60) {
                            dreheRechts();
                        } else {
                            if(istLinks("Koerner") && getHunger() < 70) {
                                dreheLinks();
                            } else {

                                if(!istVorneFrei() || istVorne("Gras") || r.nextInt(10) < 2 || (getRotation()==270 && getY()==3) || (istAuf("nest") && istVorne("nest"))) {
                                    if(istRechtsFrei() && r.nextInt(10) < 5) {
                                        dreheRechts();
                                    } else {
                                        dreheLinks();
                                    }
                                } else {
                                    einsVor();
                                }
                            }
                        }
                    }
                }
            }
        }

    }
}