import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot und MouseInfo)
import java.util.*;
import java.io.*;
/**
 * Die Klasse MyActor steht fr alle Elemente des Bauernhof
 * Jedes Element hat einen Namen, kann mehrere Bilder (Imagedatei 
 * muss mit dem Namen beginnen) haben und betretbar sein oder nicht.
 * Auerdem kann es weggenommen werden (moveable) sein oder nicht.
 * 
 * @author T. Schaller 
 * @version 04.10.2021
 */
public abstract class MyActor extends Actor
{
    protected static ArrayList<GreenfootImage> originalImages;
    private static ArrayList<String> originalImagesNames;

    protected ArrayList<Integer> imageNumbers;
    protected int bildnummer;
    protected boolean moveable;
    protected boolean betretbar;
    private String name;
    private int lastX;
    private int lastY;

    public MyActor(String name, boolean moveable, boolean betretbar) {
        if(originalImages == null) {
            originalImages = new ArrayList<GreenfootImage>();
            originalImagesNames = new ArrayList<String>();
            String[] entries = new File( "images" ).list();
            Arrays.sort(entries);
            for (String filename : entries) {

                if ((filename.toLowerCase().endsWith(".png") || filename.toLowerCase().endsWith(".jpg") || filename.toLowerCase().endsWith(".gif"))){
                    originalImages.add(new GreenfootImage("images/"+filename));        
                    originalImagesNames.add(filename);
                }
            }
        }

        imageNumbers = new ArrayList<Integer>();
        int i = 0;
        for(String filename : originalImagesNames) {
          if (filename.toUpperCase().startsWith(name.toUpperCase())){
             imageNumbers.add(i);        
          }
          i++;
        }
        this.bildnummer = 0;
        this.name = name;
        this.moveable = moveable;
        this.betretbar = betretbar;

        zeichneImage();
    }

    /** liefert, ob der Gegenstand vom Bauer aufgenommen werden kann
     * @return true, wenn der Gegenstand weggetragen werden darf.
     */
    public boolean istBeweglich() {
        return moveable;
    }

    /**
     * liefert die Information, ob das Feld (x,y) betreten werden 
     * kann.
     * @return true = wenn diese Figur das Feld nicht blockiert
     */
    public boolean istBetretbar(int x, int y) {
        return betretbar || getX() != x || getY() != y;
    }

    public String getName() {
        return name;
    }

    /** Wechsel des Bildes
     * 
     */
    protected void setzeBild(int nr) {
        if(nr>=0 && nr<imageNumbers.size()) {
            bildnummer = nr;
            zeichneImage();
        }
    }

    protected int getBild() {
        return bildnummer;
    }

    /**
     * Eine private Methode, die dafuer sorgt, dass die Figur richtig 
     * dargestellt wird.
     */
    protected void zeichneImage()
    {
        this.setImage(new GreenfootImage(originalImages.get(imageNumbers.get(bildnummer))));
        this.getImage().rotate(-getRotation());
    }

    /**
     * Wird aufgerufen, wenn der Figur in die Welt gesetzt wird; 
     * erweitert nach Vorschlag P.Henriksen 29.6.08
     * @param world Die Welt in die ein Figur gesetzt wurde
     */
    protected void addedToWorld(World world) {
        // zwei nicht betretbare Figuren knnen nicht an einem Platz stehen
        if(this.istBetretbar(getX(), getY())) return; 
        List<MyActor> list = world.getObjectsAt(getX(), getY(), MyActor.class);
        for(MyActor a: list) {
            if(a != this && !a.istBetretbar(getX(), getY())){
                melde("Hier ist kein Platz fr mich", false);
                world.removeObject(this);
                return;
            }
        }
    }

    // Meldungen und Warnungen
    /**
     * Gibt den uebergebenen String in einer Dialogbox auf dem Bildschirm aus
     * @param text Der Text, der angezeit werden soll
     * @param istWichtig bei true wird der Text "Achtung! Besonders wichtige Meldung:" in die Kopfzeile geschrieben, sonst nur der Text "Meldung!"
     */
    public void melde(String text, boolean istWichtig) {
        String meldeText;
        if (istWichtig) meldeText = "Achtung!\n";
        else meldeText = "Meldung:\n";
        Message m = new Message(meldeText+text);
        getWorld().addObject(m, 7,6);
        long z = System.currentTimeMillis();
        while(System.currentTimeMillis()-z < 4000){
            Greenfoot.delay(1);
        }
        getWorld().removeObject(m);
    }

    /**
     * Gibt den uebergebenen String und eine Zahl in einer Dialogbox auf dem Bildschirm aus
     * @param text Der Text, der angezeit werden soll
     * @param z eine Zahl, die hinter dem Text text ausgegeben wird
     */
    private void melde(String text, int z) {
        Message m = new Message(text+": "+z);
        getWorld().addObject(m, 7,6);
        long zz = System.currentTimeMillis();
        while(System.currentTimeMillis()-zz < 4000){
            Greenfoot.delay(1);
        }
        getWorld().removeObject(m);    
    }

    /**
     * Gibt den uebergebenen String in einer Dialogbox als Warnung auf dem Bildschirm aus.
     * Dadurch wird der uebergebene Actor actor von der Bauernhof entfernt und diese neu gezeichnet.
     * @param text Der Text, der angezeit werden soll
     * @param actor Der Actor, der ggf. entfernt werden soll
     */
    public void warne(String text, Actor actor) {

        Message m = new Message(text);
        actor.getWorld().addObject(m, 7,6);
        long z = System.currentTimeMillis();
        while(System.currentTimeMillis()-z < 4000){
            Greenfoot.delay(1);
        }
        actor.getWorld().removeObject(m);

        Greenfoot.stop();    // nur falls Animation laeuft
        World w = actor.getWorld();
        if (actor != null) w.removeObject(actor); // Notbremse
        Greenfoot.setWorld(new Bauernhofprojekt());
        // System.err.print("\u000C");
        // System.err.flush();
        //  System.err.close();
        // System.out.close();
        // System.out.println("Ablauf wurde vom Benutzer unterbrochen!");
        // System.exit(0); //Das waere eigentlich sauber
    }

    public int getX() {
        if(getWorld() != null) {
            lastX = super.getX();
        }
        return lastX;
    }

    public int getY() {
        if(getWorld() != null) {
            lastY = super.getY();
        }
        return lastY;
    }

    public void setLocation(int x,  int y) {
        super.setLocation(x,y);
        lastX = x;
        lastY = y;
    }
}